@@ -3,7 +3,7 @@
  * This file is part of the YATE Project http://YATE.null.ro
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2013 Null Team
+ * Copyright (C) 2004-2014 Null Team
  *
  * This software is distributed under multiple licenses;
  * see the COPYING file in the main directory for licensing
@@ -36,7 +36,7 @@ Array::Array(int columns, int rows)
 		column = column->append(a,false);
 	    // and add some empty holders for each item
 	    for (int k=1; k<rows; k++)
-		a = a->append(0,false);	
+		a = a->append(0,false);
 	}
     }
 }
@@ -62,11 +62,17 @@ bool Array::addRow(ObjList* row, int index)
     if (index > m_rows)
 	return false;
     for (int i=0; i<m_columns; i++) {
+	ObjList* list = static_cast<ObjList*>(m_obj[i]);
+	if (!list)
+	    continue;
 	GenObject* item = row ? (*row)[i] : 0;
 	if (index == m_rows)
-	    ((*(ObjList *)(m_obj[i]))+index)->append(item,false);
-	else
-	    ((*(ObjList *)(m_obj[i]))+index)->insert(item,false);
+	    list->append(item,false);
+	else {
+	    list = (*list) + index;
+	    if (list)
+		list->insert(item,false);
+	}
     }
     m_rows++;
     return true;
