@@ -3,7 +3,7 @@
  * This file is part of the YATE Project http://YATE.null.ro
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2013 Null Team
+ * Copyright (C) 2004-2014 Null Team
  *
  * This software is distributed under multiple licenses;
  * see the COPYING file in the main directory for licensing
@@ -64,12 +64,24 @@ static Mutex s_callidMutex(false,"CallID");
 // this is to protect against two threads trying to (dis)connect a pair
 //  of call endpoints at the same time
 static Mutex s_mutex(true,"CallEndpoint");
+static Mutex s_lastMutex(false,"CallEndpoint::last");
 static const String s_audioType = "audio";
 static const String s_copyParams = "copyparams";
 
+// Check if a Lock taken on the common mutex succeeded, wait up to 55s more in congestion
+static bool checkRetry(Lock& lock)
+{
+    if (lock.locked())
+	return true;
+    Engine::setCongestion("Call endpoint mutex busy");
+    bool ok = lock.acquire(s_mutex,55000000);
+    Engine::setCongestion();
+    return ok;
+}
+
 
 CallEndpoint::CallEndpoint(const char* id)
-    : m_peer(0), m_id(id), m_mutex(0)
+    : m_peer(0), m_lastPeer(0), m_id(id), m_mutex(0)
 {
 }
 
@@ -84,6 +96,7 @@ void CallEndpoint::destroyed()
 #endif
     disconnect(true,0,true,0);
     clearEndpoint();
+    m_lastPeer = 0;
 }
 
 Mutex& CallEndpoint::commonMutex()
@@ -119,7 +132,7 @@ bool CallEndpoint::connect(CallEndpoint* peer, const char* reason, bool notify)
 
 #if 0
     Lock lock(s_mutex,5000000);
-    if (!lock.locked()) {
+    if (!checkRetry(lock)) {
 	Alarm("engine","bug",DebugFail,"Call connect failed - timeout on call endpoint mutex owned by '%s'!",s_mutex.owner());
 	Engine::restart(0);
 	return false;
@@ -158,7 +171,7 @@ bool CallEndpoint::disconnect(bool final, const char* reason, bool notify, const
     DDebug(DebugAll,"CallEndpoint '%s' disconnecting peer %p from [%p]",m_id.c_str(),m_peer,this);
 
     Lock lock(s_mutex,5000000);
-    if (!lock.locked()) {
+    if (!checkRetry(lock)) {
 	Alarm("engine","bug",DebugFail,"Call disconnect failed - timeout on call endpoint mutex owned by '%s'!",s_mutex.owner());
 	Engine::restart(0);
 	return false;
@@ -166,6 +179,7 @@ bool CallEndpoint::disconnect(bool final, const char* reason, bool notify, const
 
     CallEndpoint *temp = m_peer;
     m_peer = 0;
+    m_lastPeer = 0;
     if (!temp)
 	return false;
 
@@ -191,9 +205,12 @@ void CallEndpoint::setPeer(CallEndpoint* peer, const char* reason, bool notify,
 	setDisconnect(0);
 	connected(reason);
     }
-    else if (notify) {
-	setDisconnect(params);
-	disconnected(false,reason);
+    else {
+	m_lastPeer = 0;
+	if (notify) {
+	    setDisconnect(params);
+	    disconnected(false,reason);
+	}
     }
 }
 
@@ -202,8 +219,15 @@ bool CallEndpoint::getPeerId(String& id) const
     id.clear();
     if (!m_peer)
 	return false;
+    if (m_peer == m_lastPeer) {
+	Lock mylock(s_lastMutex);
+	if (m_peer == m_lastPeer) {
+	    id = m_lastPeerId;
+	    return !id.null();
+	}
+    }
     Lock lock(s_mutex,5000000);
-    if (!lock.locked()) {
+    if (!checkRetry(lock)) {
 	Alarm("engine","bug",DebugFail,"Peer ID failed - timeout on call endpoint mutex owned by '%s'!",s_mutex.owner());
 	Engine::restart(0);
 	return false;
@@ -223,6 +247,36 @@ String CallEndpoint::getPeerId() const
     return id;
 }
 
+bool CallEndpoint::getLastPeerId(String& id) const
+{
+    id.clear();
+    if (m_lastPeerId.null())
+	return false;
+    s_lastMutex.lock();
+    id = m_lastPeerId;
+    s_lastMutex.unlock();
+    return !id.null();
+}
+
+void CallEndpoint::setLastPeerId()
+{
+    if (!m_peer)
+	return;
+    if (m_peer == m_lastPeer)
+	return;
+    Lock lock(s_mutex,5000000);
+    if (!checkRetry(lock)) {
+	Alarm("engine","bug",DebugGoOn,"Set last peer ID failed - timeout on call endpoint mutex owned by '%s'!",s_mutex.owner());
+	return;
+    }
+    if (m_peer) {
+	s_lastMutex.lock();
+	m_lastPeer = m_peer;
+	m_lastPeerId = m_peer->id();
+	s_lastMutex.unlock();
+    }
+}
+
 DataEndpoint* CallEndpoint::getEndpoint(const String& type) const
 {
     if (type.null())
@@ -460,11 +514,11 @@ void Channel::connected(const char* reason)
 	    m_billid = peer->billid();
     }
     Message* m = message("chan.connected",false,true);
+    setLastPeerId();
     if (reason)
 	m->setParam("reason",reason);
     if (!Engine::enqueue(m))
 	TelEngine::destruct(m);
-    getPeerId(m_lastPeerId);
 }
 
 void Channel::disconnected(bool final, const char* reason)
@@ -607,8 +661,8 @@ void Channel::complete(Message& msg, bool minimal) const
     String peer;
     if (getPeerId(peer))
 	msg.setParam("peerid",peer);
-    if (m_lastPeerId)
-	msg.setParam("lastpeerid",m_lastPeerId);
+    if (getLastPeerId(peer))
+	msg.setParam("lastpeerid",peer);
     msg.setParam("answered",String::boolText(m_answered));
     msg.setParam("direction",direction());
 }
@@ -1031,6 +1085,7 @@ bool Module::installRelay(int id, const char* name, unsigned priority)
     if (!(id && name && priority))
 	return false;
 
+    TempObjectCounter cnt(objectsCounter(),true);
     Lock lock(this);
     if (m_relays & id)
 	return true;
@@ -1244,6 +1299,7 @@ bool Module::setDebug(Message& msg, const String& target)
     if (target != name())
 	return false;
 
+    NamedCounter* counter = objectsCounter();
     String str = msg.getValue("line");
     if (str.startSkip("level")) {
 	int dbg = debugLevel();
@@ -1253,6 +1309,14 @@ bool Module::setDebug(Message& msg, const String& target)
     else if (str == "reset") {
 	debugLevel(TelEngine::debugLevel());
 	debugEnabled(true);
+	if (counter)
+	    counter->enable(getObjCounting());
+    }
+    else if (str.startSkip("objects")) {
+	bool dbg = (str == "reset") ? getObjCounting() : (counter && counter->enabled());
+	str >> dbg;
+	if (counter)
+	    counter->enable(dbg);
     }
     else if (str.startSkip("filter"))
 	m_filter = str;
@@ -1263,7 +1327,8 @@ bool Module::setDebug(Message& msg, const String& target)
     }
     msg.retValue() << "Module " << name()
 	<< " debug " << (debugEnabled() ? "on" : "off")
-	<< " level " << debugLevel();
+	<< " level " << debugLevel()
+	<< " objects " << ((counter && counter->enabled()) ? "on" : "off");
     if (m_filter)
 	msg.retValue() << " filter: " << m_filter;
     msg.retValue() << "\r\n";
@@ -1357,11 +1422,13 @@ bool Driver::received(Message &msg, int id)
 		    lock();
 		}
 	    }
+	    return Module::received(msg,id);
 	case Status:
 	    // check if it's a channel status request
 	    dest = msg.getValue(YSTRING("module"));
 	    if (dest.startsWith(m_prefix))
 		break;
+	    // fall through
 	case Level:
 	case Route:
 	case Command:
@@ -1484,7 +1551,7 @@ bool Driver::canAccept(bool routers)
 
 bool Driver::canRoute()
 {
-    if (Engine::exiting())
+    if (Engine::exiting() || (Engine::accept() >= Engine::Congestion))
 	return false;
     if (m_maxroute && (m_routing >= m_maxroute))
 	return false;
@@ -1606,6 +1673,8 @@ unsigned int Driver::nextid()
 Router::Router(Driver* driver, const char* id, Message* msg)
     : Thread("Call Router"), m_driver(driver), m_id(id), m_msg(msg)
 {
+    if (driver)
+	setObjCounter(driver->objectsCounter());
 }
 
 void Router::run()
