@@ -3,7 +3,7 @@
  * This file is part of the YATE Project http://YATE.null.ro
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2013 Null Team
+ * Copyright (C) 2004-2014 Null Team
  *
  * Idea and initial implementation (as HashTable) by Maciek Kaminski
  *
@@ -104,9 +104,13 @@ ObjList* HashList::append(const GenObject* obj)
     return m_lists[i]->append(obj);
 }
 
-GenObject* HashList::remove(GenObject* obj, bool delobj)
+GenObject* HashList::remove(GenObject* obj, bool delobj, bool useHash)
 {
-    ObjList *n = find(obj);
+    ObjList* n = 0;
+    if (useHash && obj)
+	n = find(obj,obj->toString().hash());
+    else
+	n = find(obj);
     return n ? n->remove(delobj) : 0;
 }
 
