@@ -3,7 +3,7 @@
  * This file is part of the YATE Project http://YATE.null.ro
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2013 Null Team
+ * Copyright (C) 2004-2014 Null Team
  *
  * This software is distributed under multiple licenses;
  * see the COPYING file in the main directory for licensing
@@ -237,10 +237,23 @@ SocketAddr::SocketAddr(const struct sockaddr* addr, socklen_t len)
     assign(addr,len);
 }
 
-SocketAddr::SocketAddr(int family)
+SocketAddr::SocketAddr(int family, const void* raw)
     : m_address(0), m_length(0)
 {
     assign(family);
+    if (raw && m_address) {
+	switch (family) {
+	    case AF_INET:
+		::memcpy(&((struct sockaddr_in*)m_address)->sin_addr,raw,4);
+		break;
+#ifdef AF_INET6
+	    case AF_INET6:
+		::memcpy(&((struct sockaddr_in6*)m_address)->sin6_addr,raw,16);
+		break;
+#endif
+	}
+	SocketAddr::stringify();
+    }
 }
 
 SocketAddr::~SocketAddr()
@@ -609,7 +622,7 @@ void SocketAddr::split(const String& buf, String& addr, int& port, bool portPres
 	    port = buf.substr(p2 + 1).toInteger();
 	    addr.assign(buf.c_str(),p2);
 	}
-	else 
+	else
 	    addr = buf;
     }
     else
@@ -1161,7 +1174,7 @@ int File::readData(void* buffer, int length)
     else if (::GetLastError() == ERROR_HANDLE_EOF) {
 	clearError();
 	return 0;
-    } 
+    }
     copyError();
     return -1;
 #else
