@@ -3,7 +3,7 @@
  * This file is part of the YATE Project http://YATE.null.ro
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2013 Null Team
+ * Copyright (C) 2004-2014 Null Team
  *
  * This software is distributed under multiple licenses;
  * see the COPYING file in the main directory for licensing
@@ -1350,12 +1350,11 @@ String String::uriUnescape(const char* str, int* errptr)
     return s;
 }
 
-unsigned int String::hash(const char* value)
+unsigned int String::hash(const char* value, unsigned int h)
 {
     if (!value)
 	return 0;
 
-    unsigned int h = 0;
     // sdbm hash algorithm, hash(i) = hash(i-1) * 65599 + str[i]
     while (unsigned char c = (unsigned char) *value++)
 	h = (h << 6) + (h << 16) - h + c;
