@@ -1,9 +1,10 @@
 /**
- * Evaluator.cpp
+ * evaluator.cpp
+ * Yet Another (Java)script library
  * This file is part of the YATE Project http://YATE.null.ro
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2013 Null Team
+ * Copyright (C) 2004-2014 Null Team
  *
  * This software is distributed under multiple licenses;
  * see the COPYING file in the main directory for licensing
@@ -340,6 +341,8 @@ bool ExpEvaluator::getOperandInternal(ParsePoint& expr, bool endOk, int preceden
 	expr++;
 	return true;
     }
+    if (getNumber(expr))
+	return true;
     Opcode op = getUnaryOperator(expr);
     if (op != OpcNone) {
 	if (!getOperand(expr,false,getPrecedence(op)))
@@ -1016,10 +1019,11 @@ bool ExpEvaluator::runOperation(ObjList& stack, const ExpOperation& oper, GenObj
 		switch (oper.opcode()) {
 		    case OpcDiv:
 		    case OpcMod:
-			if (!op2->valInteger())
+			if (!op2->toNumber())
 			    return gotError("Division by zero",oper.lineNumber());
+			break;
 		    case OpcAdd:
-			if (op1->isInteger() && op2->isInteger())
+			if (op1->isNumber() && op2->isNumber())
 			    break;
 			// turn addition into concatenation
 			{
@@ -1034,6 +1038,7 @@ bool ExpEvaluator::runOperation(ObjList& stack, const ExpOperation& oper, GenObj
 			break;
 		}
 		int64_t val = 0;
+		bool handled = true;
 		switch (oper.opcode()) {
 		    case OpcAnd:
 			val = op1->valInteger() & op2->valInteger();
@@ -1050,21 +1055,6 @@ bool ExpEvaluator::runOperation(ObjList& stack, const ExpOperation& oper, GenObj
 		    case OpcShr:
 			val = op1->valInteger() >> op2->valInteger();
 			break;
-		    case OpcAdd:
-			val = op1->valInteger() + op2->valInteger();
-			break;
-		    case OpcSub:
-			val = op1->valInteger() - op2->valInteger();
-			break;
-		    case OpcMul:
-			val = op1->valInteger() * op2->valInteger();
-			break;
-		    case OpcDiv:
-			val = op1->valInteger() / op2->valInteger();
-			break;
-		    case OpcMod:
-			val = op1->valInteger() % op2->valInteger();
-			break;
 		    case OpcLt:
 			val = (op1->valInteger() < op2->valInteger()) ? 1 : 0;
 			break;
@@ -1078,14 +1068,48 @@ bool ExpEvaluator::runOperation(ObjList& stack, const ExpOperation& oper, GenObj
 			val = (op1->valInteger() >= op2->valInteger()) ? 1 : 0;
 			break;
 		    case OpcEq:
-			val = (*op1 == *op2) ? 1 : 0;
-			break;
 		    case OpcNe:
-			val = (*op1 != *op2) ? 1 : 0;
+		    {
+			ExpWrapper* w1 = YOBJECT(ExpWrapper,op1);
+			ExpWrapper* w2 = YOBJECT(ExpWrapper,op2);
+			if (op1->opcode() == op2->opcode() && w1 && w2)
+			    val = w1->object() == w2->object() ? 1 : 0;
+			else
+			    val = (*op1 == *op2) ? 1 : 0;
+			if (oper.opcode() == OpcNe)
+			    val = val ? 0 : 1;
 			break;
+		    }
 		    default:
+			handled = false;
 			break;
 		}
+		if (!handled) {
+		    val = ExpOperation::nonInteger();
+		    int64_t op1Val = op1->toNumber();
+		    int64_t op2Val = op2->toNumber();
+		    if (op1Val != ExpOperation::nonInteger() && op2Val != ExpOperation::nonInteger()) {
+			switch(oper.opcode()) {
+			    case OpcAdd:
+				val = op1Val + op2Val;
+				break;
+			    case OpcSub:
+				val = op1Val - op2Val;
+				break;
+			    case OpcMul:
+				val = op1Val * op2Val;
+				break;
+			    case OpcDiv:
+				val = op1Val / op2Val;
+				break;
+			    case OpcMod:
+				val = op1Val % op2Val;
+				break;
+			    default:
+				break;
+			}
+		    }
+		}
 		TelEngine::destruct(op1);
 		TelEngine::destruct(op2);
 		if (boolRes) {
@@ -1164,7 +1188,7 @@ bool ExpEvaluator::runOperation(ObjList& stack, const ExpOperation& oper, GenObj
 		    return gotError("ExpEvaluator stack underflow",oper.lineNumber());
 		switch (oper.opcode()) {
 		    case OpcNeg:
-			pushOne(stack,new ExpOperation(-op->valInteger()));
+			pushOne(stack,new ExpOperation(-op->toNumber()));
 			break;
 		    case OpcNot:
 			pushOne(stack,new ExpOperation(~op->valInteger()));
@@ -1463,7 +1487,7 @@ void ExpEvaluator::dump(const ExpOperation& oper, String& res) const
 	        const char* name = getOperator(oper.opcode());
 		if (name)
 		    res << name;
-		else 
+		else
 		    res << "[" << oper.opcode() << "]";
 	    }
 	    if (oper.number() && oper.isInteger())
@@ -1491,6 +1515,13 @@ int64_t ExpOperation::valInteger() const
     return isInteger() ? number() : 0;
 }
 
+int64_t  ExpOperation::toNumber() const
+{
+    if (isInteger())
+	return number();
+    return toInt64(nonInteger());
+}
+
 bool ExpOperation::valBoolean() const
 {
     return isInteger() ? (number() != 0) : !null();
@@ -1501,7 +1532,7 @@ const char* ExpOperation::typeOf() const
     switch (opcode()) {
 	case ExpEvaluator::OpcPush:
 	case ExpEvaluator::OpcCopy:
-	    return isInteger() ? "number" : "string";
+	    return isInteger() ? ( isBoolean() ? "boolean" : "number" ) : (isNumber() ? "number" : "string");
 	case ExpEvaluator::OpcFunc:
 	    return "function";
 	default:
