@@ -4,7 +4,7 @@
  * This file is part of the YATE Project http://YATE.null.ro
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2011-2013 Null Team
+ * Copyright (C) 2011-2014 Null Team
  *
  * This software is distributed under multiple licenses;
  * see the COPYING file in the main directory for licensing
@@ -70,6 +70,8 @@ public:
 	{ return false; }
     virtual ExpOperation* clone(const char* name) const
 	{ return new ExpNull(static_cast<JsNull*>(object()),name); }
+    virtual ExpOperation* copy(Mutex* mtx) const
+	{ return clone(name()); }
 protected:
     inline ExpNull(JsNull* obj, const char* name)
 	: ExpWrapper(obj,name)
@@ -172,8 +174,8 @@ public:
     bool link();
     inline bool traceable() const
 	{ return m_traceable; }
-    JsObject* parseArray(ParsePoint& expr, bool constOnly);
-    JsObject* parseObject(ParsePoint& expr, bool constOnly);
+    JsObject* parseArray(ParsePoint& expr, bool constOnly, Mutex* mtx);
+    JsObject* parseObject(ParsePoint& expr, bool constOnly, Mutex* mtx);
     inline const NamedList& pragmas() const
 	{ return m_pragmas; }
     inline static unsigned int getLineNo(unsigned int line)
@@ -225,6 +227,7 @@ private:
     bool parseVar(ParsePoint& expr);
     bool parseTry(ParsePoint& expr, GenObject* nested);
     bool parseFuncDef(ParsePoint& expr, bool publish);
+    bool parseSimple(ParsePoint& expr, bool constOnly, Mutex* mtx = 0);
     bool evalList(ObjList& stack, GenObject* context) const;
     bool evalVector(ObjList& stack, GenObject* context) const;
     bool jumpToLabel(long int label, GenObject* context) const;
@@ -729,7 +732,7 @@ bool JsContext::runStringFunction(GenObject* obj, const String& name, ObjList& s
 		ok = buf.matches(r);
 	    }
 	    if (ok) {
-		JsArray* jsa = new JsArray(mutex());
+		JsArray* jsa = new JsArray(context,mutex());
 		for (int i = 0; i <= buf.matchCount(); i++)
 		    jsa->push(new ExpOperation(buf.matchString(i)));
 		jsa->params().addParam(new ExpOperation((int64_t)buf.matchOffset(),"index"));
@@ -784,7 +787,7 @@ bool JsContext::runStringFunction(GenObject* obj, const String& name, ObjList& s
 		    pos = tmp->toInteger(0); \
 	    } \
 	} \
-	String s(*str); 
+	String s(*str);
 
     if (name == YSTRING("startsWith")) {
 	MAKE_WITH_METHOD;
@@ -798,7 +801,7 @@ bool JsContext::runStringFunction(GenObject* obj, const String& name, ObjList& s
 	if (pos > 0)
 	    s = s.substr(0,pos);
 	ExpEvaluator::pushOne(stack,new ExpOperation(s.endsWith(what)));
-	return true; 
+	return true;
     }
 #undef MAKE_WITH_METHOD
 #define SPLIT_EMPTY() do { \
@@ -808,7 +811,7 @@ bool JsContext::runStringFunction(GenObject* obj, const String& name, ObjList& s
     } while (false);
     if (name == YSTRING("split")) {
 	ObjList args;
-	JsArray* array = new JsArray(mutex());
+	JsArray* array = new JsArray(context,mutex());
 	if (!(extractArgs(stack,oper,context,args) && args.skipNull()))
 	    SPLIT_EMPTY();
 	String* s = static_cast<String*>(args[0]);
@@ -839,6 +842,10 @@ bool JsContext::runStringFunction(GenObject* obj, const String& name, ObjList& s
 	extractArgs(stack,oper,context,args);
 	ExpOperation* op = YOBJECT(ExpOperation,str);
 	if (op && op->isInteger()) {
+	    if (op->isBoolean()) {
+		ExpEvaluator::pushOne(stack,new ExpOperation(String::boolText(op->valBoolean())));
+		return true;
+	    }
 	    ExpOperation* tmp = static_cast<ExpOperation*>(args[0]);
 	    int radix = tmp ? (int)tmp->valInteger() : 0;
 	    if (radix < 2 || radix > 36)
@@ -1215,7 +1222,7 @@ bool JsCode::preProcessInclude(ParsePoint& expr, bool once, GenObject* context)
 	String str;
 	if (ExpEvaluator::getString(expr,str)) {
 	    DDebug(this,DebugAll,"Found include '%s'",str.safe());
-	    parser->adjustPath(str);
+	    parser->adjustPath(str,true);
 	    str.trimSpaces();
 	    bool ok = !str.null();
 	    if (ok) {
@@ -1365,6 +1372,8 @@ bool JsCode::getInstruction(ParsePoint& expr, char stop, GenObject* nested)
 	    addOpcode(op);
 	    break;
 	case OpcReturn:
+	{
+	    int64_t pop = ExpOperation::nonInteger();
 	    switch (skipComments(expr)) {
 		case ';':
 		case '}':
@@ -1372,11 +1381,13 @@ bool JsCode::getInstruction(ParsePoint& expr, char stop, GenObject* nested)
 		default:
 		    if (!runCompile(expr,';'))
 			return false;
+		    pop = 1;
 		    if ((skipComments(expr) != ';') && (*expr != '}'))
 			return gotError("Expecting ';' or '}'",expr);
 	    }
-	    addOpcode(op);
+	    addOpcode(op,pop);
 	    break;
+	}
 	case OpcIf:
 	    return parseIf(expr,nested);
 	case OpcElse:
@@ -1779,6 +1790,7 @@ bool JsCode::parseFuncDef(ParsePoint& expr, bool publish)
     if (*expr != '}')
 	return gotError("Expecting '}'",expr);
     expr++;
+    // Add the implicit "return undefined" at end of function
     addOpcode((Opcode)OpcReturn);
     addOpcode(OpcLabel,jump->number());
     JsFunction* obj = new JsFunction(0,name,&args,(long int)lbl->number(),this);
@@ -1897,10 +1909,15 @@ bool JsCode::getSeparator(ParsePoint& expr, bool remove)
 }
 
 bool JsCode::getSimple(ParsePoint& expr, bool constOnly)
+{
+    return parseSimple(expr,constOnly);
+}
+
+bool JsCode::parseSimple(ParsePoint& expr, bool constOnly, Mutex* mtx)
 {
     if (inError())
 	return false;
-    XDebug(this,DebugAll,"JsCode::getSimple(%s) '%.30s'",String::boolText(constOnly),(const char*)expr);
+    XDebug(this,DebugAll,"JsCode::parseSimple(%s) '%.30s'",String::boolText(constOnly),(const char*)expr);
     skipComments(expr);
     ParsePoint save = expr;
     switch ((JsOpcode)ExpEvaluator::getOperator(expr,s_constants)) {
@@ -1925,9 +1942,9 @@ bool JsCode::getSimple(ParsePoint& expr, bool constOnly)
 	default:
 	    break;
     }
-    JsObject* jso = parseArray(expr,constOnly);
+    JsObject* jso = parseArray(expr,constOnly,mtx);
     if (!jso)
-	jso = parseObject(expr,constOnly);
+	jso = parseObject(expr,constOnly,mtx);
     if (!jso)
 	return ExpEvaluator::getSimple(expr,constOnly);
     addOpcode(new ExpWrapper(ExpEvaluator::OpcCopy,jso));
@@ -1935,25 +1952,47 @@ bool JsCode::getSimple(ParsePoint& expr, bool constOnly)
 }
 
 // Parse an inline Javascript Array: [ item1, item2, ... ]
-JsObject* JsCode::parseArray(ParsePoint& expr, bool constOnly)
+JsObject* JsCode::parseArray(ParsePoint& expr, bool constOnly, Mutex* mtx)
 {
     if (skipComments(expr) != '[')
 	return 0;
     expr++;
-    JsArray* jsa = new JsArray(0,"[object Array]");
+    JsArray* jsa = new JsArray(mtx,"[object Array]");
     for (bool first = true; ; first = false) {
 	if (skipComments(expr) == ']') {
 	    expr++;
 	    break;
 	}
-	if (!first) {
+	if (first) {
+	    if (*expr == ',') {
+		ParsePoint next = expr;
+		next++;
+		// A construct like [,] creates an empty Array
+		if (skipComments(next) == ']') {
+		    next++;
+		    expr = next;
+		    break;
+		}
+	    }
+	}
+	else {
 	    if (*expr != ',') {
 		TelEngine::destruct(jsa);
 		break;
 	    }
 	    expr++;
 	}
-	bool ok = constOnly ? getSimple(expr,true) : getOperand(expr,false);
+	// Swallow the single comma allowed after last item
+	if (skipComments(expr) == ']') {
+	    expr++;
+	    break;
+	}
+	// Successive commas insert an undefined between them
+	if (skipComments(expr) == ',') {
+	    jsa->push(new ExpWrapper(0,"undefined"));
+	    continue;
+	}
+	bool ok = constOnly ? parseSimple(expr,true,mtx) : getOperand(expr,false);
 	if (!ok) {
 	    TelEngine::destruct(jsa);
 	    break;
@@ -1968,12 +2007,12 @@ JsObject* JsCode::parseArray(ParsePoint& expr, bool constOnly)
 
 
 // Parse an inline Javascript Object: { prop1: value1, "prop 2": value2, ... }
-JsObject* JsCode::parseObject(ParsePoint& expr, bool constOnly)
+JsObject* JsCode::parseObject(ParsePoint& expr, bool constOnly, Mutex* mtx)
 {
     if (skipComments(expr) != '{')
 	return 0;
     expr++;
-    JsObject* jso = new JsObject(0,"[object Object]");
+    JsObject* jso = new JsObject(mtx,"[object Object]");
     for (bool first = true; ; first = false) {
 	if (skipComments(expr) == '}') {
 	    expr++;
@@ -1985,6 +2024,11 @@ JsObject* JsCode::parseObject(ParsePoint& expr, bool constOnly)
 		break;
 	    }
 	    expr++;
+	    // A single comma is allowed after last property
+	    if (skipComments(expr) == '}') {
+		expr++;
+		break;
+	    }
 	}
 	char c = skipComments(expr);
 	String name;
@@ -2002,7 +2046,7 @@ JsObject* JsCode::parseObject(ParsePoint& expr, bool constOnly)
 	    break;
 	}
 	expr++;
-	bool ok = constOnly ? getSimple(expr,true) : getOperand(expr,false);
+	bool ok = constOnly ? parseSimple(expr,true,mtx) : getOperand(expr,false);
 	if (!ok) {
 	    TelEngine::destruct(jso);
 	    break;
@@ -2226,7 +2270,7 @@ bool JsCode::runOperation(ObjList& stack, const ExpOperation& oper, GenObject* c
 	    break;
 	case OpcReturn:
 	    {
-		ExpOperation* op = popValue(stack,context);
+		ExpOperation* op = oper.valInteger() ? popValue(stack,context) : new ExpWrapper(0,"undefined");
 		ExpOperation* thisObj = 0;
 		bool ok = false;
 		while (ExpOperation* drop = popAny(stack)) {
@@ -2498,6 +2542,8 @@ void JsCode::resolveObjectParams(JsObject* object, ObjList& stack, GenObject* co
 	    temp = new NamedString(op->name(),*ns);
 	object->params().setParam(temp);
     }
+    if (object->frozen())
+	return;
     JsArray* arr = YOBJECT(JsArray,object);
     if (arr) {
 	if (arrayProto && arrayProto->ref())
@@ -3364,11 +3410,14 @@ bool JsFunction::runDefined(ObjList& stack, const ExpOperation& oper, GenObject*
 
 
 // Adjust a script file include path
-void JsParser::adjustPath(String& script) const
+void JsParser::adjustPath(String& script, bool extraInc) const
 {
     if (script.null() || script.startsWith(Engine::pathSeparator()))
 	return;
-    script = m_basePath + script;
+    if (extraInc && m_includePath && File::exists(m_includePath + script))
+	script = m_includePath + script;
+    else
+	script = m_basePath + script;
 }
 
 // Create Javascript context
@@ -3462,13 +3511,17 @@ ScriptRun::Status JsParser::eval(const String& text, ExpOperation** result, Scri
 }
 
 // Parse JSON using native methods
-JsObject* JsParser::parseJSON(const char* text)
+ExpOperation* JsParser::parseJSON(const char* text, Mutex* mtx)
 {
+    if (!text)
+	return 0;
+    ExpOperation* ret = 0;
     JsCode* code = new JsCode;
     ParsePoint pp(text,code);
-    JsObject* jso = code->parseObject(pp,true);
+    if (code->parseSimple(pp,true,mtx))
+	ret = code->popOpcode();
     TelEngine::destruct(code);
-    return jso;
+    return ret;
 }
 
 // Return a "null" object wrapper
@@ -3477,6 +3530,13 @@ ExpOperation* JsParser::nullClone(const char* name)
     return TelEngine::null(name) ? s_null.ExpOperation::clone() : s_null.clone(name);
 }
 
+// Return the "null" object
+JsObject* JsParser::nullObject()
+{
+    JsObject* n = YOBJECT(JsObject,s_null.object());
+    return (n && n->ref()) ? n : 0;
+}
+
 // Check if an object is identic to null
 bool JsParser::isNull(const ExpOperation& oper)
 {
