@@ -4,7 +4,7 @@
  * This file is part of the YATE Project http://YATE.null.ro
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2011-2013 Null Team
+ * Copyright (C) 2011-2014 Null Team
  *
  * This software is distributed under multiple licenses;
  * see the COPYING file in the main directory for licensing
@@ -124,6 +124,7 @@ static void dumpRecursiveObj(const GenObject* obj, String& buf, unsigned int dep
     const NamedString* nstr = YOBJECT(NamedString,obj);
     const NamedPointer* nptr = YOBJECT(NamedPointer,nstr);
     const char* type = nstr ? (nptr ? "NamedPointer" : "NamedString") : "???";
+    const char* subType = 0;
     const ScriptContext* scr = YOBJECT(ScriptContext,obj);
     const ExpWrapper* wrap = 0;
     bool objRecursed = false;
@@ -152,14 +153,16 @@ static void dumpRecursiveObj(const GenObject* obj, String& buf, unsigned int dep
 	    type = wrap->object() ? "ExpWrapper" : "Undefined";
 	else if (YOBJECT(ExpFunction,exp))
 	    type = "ExpFunction";
-	else
+	else {
 	    type = "ExpOperation";
+	    subType = exp->typeOf();
+	}
     }
     if (nstr)
 	str << "'" << nstr->name() << "' = '" << *nstr << "'";
     else
 	str << "'" << obj->toString() << "'";
-    str << " (" << type << ")";
+    str << " (" << type << (subType ? ", " : "") << subType << ")";
     if (objRecursed)
 	str << " (already seen)";
     buf.append(str,"\r\n");
@@ -205,6 +208,13 @@ JsObject::JsObject(Mutex* mtx, const char* name, bool frozen)
 	mtx,name,String::boolText(frozen),this);
 }
 
+JsObject::JsObject(GenObject* context, Mutex* mtx, bool frozen)
+    : ScriptContext("[object Object]"),
+      m_frozen(frozen), m_mutex(mtx)
+{
+    setPrototype(context,YSTRING("Object"));
+}
+
 JsObject::~JsObject()
 {
     XDebug(DebugAll,"JsObject::~JsObject '%s' [%p]",toString().c_str(),this);
@@ -230,6 +240,22 @@ void JsObject::printRecursive(const GenObject* obj)
     Output("%s",buf.c_str());
 }
 
+void JsObject::setPrototype(GenObject* context, const String& objName)
+{
+    ScriptContext* ctxt = YOBJECT(ScriptContext,context);
+    if (!ctxt) {
+	ScriptRun* sr = static_cast<ScriptRun*>(context);
+	if (!(sr && (ctxt = YOBJECT(ScriptContext,sr->context()))))
+	    return;
+    }
+    JsObject* objCtr = YOBJECT(JsObject,ctxt->params().getParam(objName));
+    if (objCtr) {
+	JsObject* proto = YOBJECT(JsObject,objCtr->params().getParam(YSTRING("prototype")));
+	if (proto && proto->ref())
+	    params().addParam(new ExpWrapper(proto,protoName()));
+    }
+}
+
 JsObject* JsObject::buildCallContext(Mutex* mtx, JsObject* thisObj)
 {
     JsObject* ctxt = new JsObject(mtx,"()");
@@ -322,11 +348,13 @@ bool JsObject::runField(ObjList& stack, const ExpOperation& oper, GenObject* con
 	    if (w)
 		ExpEvaluator::pushOne(stack,w->clone(oper.name()));
 	    else {
-		bool num = true;
-		ExpOperation* o = YOBJECT(ExpOperation,param);
-		if (o && !o->isInteger())
-		    num = false;
-		ExpEvaluator::pushOne(stack,new ExpOperation(*param,oper.name(),num));
+		JsObject* jso = YOBJECT(JsObject,param);
+		if (jso && jso->ref())
+		    ExpEvaluator::pushOne(stack,new ExpWrapper(jso,oper.name()));
+		else {
+		    ExpOperation* o = YOBJECT(ExpOperation,param);
+		    ExpEvaluator::pushOne(stack,o ? new ExpOperation(*o,oper.name(),false) : new ExpOperation(*param,oper.name(),true));
+		}
 	    }
 	}
     }
@@ -490,6 +518,12 @@ JsArray::JsArray(Mutex* mtx)
     params().addParam("length","0");
 }
 
+JsArray::JsArray(GenObject* context, Mutex* mtx)
+    : JsObject(mtx,"[object Array]"), m_length(0)
+{
+    setPrototype(context,YSTRING("Array"));
+}
+
 JsObject* JsArray::copy(Mutex* mtx) const
 {
     JsArray* jsa = new JsArray(mtx,toString(),frozen());
@@ -543,11 +577,43 @@ bool JsArray::runField(ObjList& stack, const ExpOperation& oper, GenObject* cont
     return JsObject::runField(stack,oper,context);
 }
 
+void JsArray::initConstructor(JsFunction* construct)
+{
+    construct->params().addParam(new ExpFunction("isArray"));
+}
+
+JsObject* JsArray::runConstructor(ObjList& stack, const ExpOperation& oper, GenObject* context)
+{
+    if (!ref())
+	return 0;
+    JsArray* obj = static_cast<JsArray*>(clone("[object " + oper.name() + "]"));
+    unsigned int len = oper.number();
+    for (unsigned int i = len; i;  i--) {
+	ExpOperation* op = obj->popValue(stack,context);
+	if ((len == 1) && op->isInteger() && (op->number() >= 0) && (op->number() <= 0xffffffff)) {
+	    len = op->number();
+	    TelEngine::destruct(op);
+	    break;
+	}
+	const_cast<String&>(op->name()) = i - 1;
+	obj->params().paramList()->insert(op);
+    }
+    obj->setLength(len);
+    obj->params().addParam(new ExpWrapper(this,protoName()));
+    return obj;
+}
+
 bool JsArray::runNative(ObjList& stack, const ExpOperation& oper, GenObject* context)
 {
     XDebug(DebugAll,"JsArray::runNative() '%s' in '%s' [%p]",
 	oper.name().c_str(),toString().c_str(),this);
-    if (oper.name() == YSTRING("push")) {
+    if (oper.name() == YSTRING("isArray")) {
+	// Static function that checks if the argument is an Array
+	ObjList args;
+	extractArgs(this,stack,oper,context,args);
+	ExpEvaluator::pushOne(stack,new ExpOperation(YOBJECT(JsArray,args[0])));
+    }
+    else if (oper.name() == YSTRING("push")) {
 	// Adds one or more elements to the end of an array and returns the new length of the array.
 	ObjList args;
 	if (!extractArgs(this,stack,oper,context,args))
@@ -560,71 +626,68 @@ bool JsArray::runNative(ObjList& stack, const ExpOperation& oper, GenObject* con
     }
     else if (oper.name() == YSTRING("pop")) {
 	// Removes the last element from an array and returns that element
-	if (m_length < 1)
-	    ExpEvaluator::pushOne(stack,new ExpWrapper(0,0));
-
+	if (oper.number())
+	    return false;
 	NamedString* last = 0;
-	while (!last) {
+	while ((m_length > 0) && !last)
 	    last = params().getParam(String(--m_length));
-	    if (m_length == 0)
-		break;
-	}
 	if (!last)
 	    ExpEvaluator::pushOne(stack,new ExpWrapper(0,0));
 	else {
-	    ExpWrapper* w = YOBJECT(ExpWrapper,last);
-	    if (!w)
-		ExpEvaluator::pushOne(stack,new ExpOperation(*last));
-	    else
-		ExpEvaluator::pushOne(stack,w->clone(w->name()));
+	    params().paramList()->remove(last,false);
+	    ExpOperation* op = YOBJECT(ExpOperation,last);
+	    if (!op) {
+		op = new ExpOperation(*last,0,true);
+		TelEngine::destruct(last);
+	    }
+	    ExpEvaluator::pushOne(stack,op);
 	}
-	// clear last
-	params().clearParam(last);
     }
     else if (oper.name() == YSTRING("concat")) {
 	// Returns a new array comprised of this array joined with other array(s) and/or value(s).
-	// var num1 = [1, 2, 3];  
-	// var num2 = [4, 5, 6];  
-	// var num3 = [7, 8, 9];  
+	// var num1 = [1, 2, 3];
+	// var num2 = [4, 5, 6];
+	// var num3 = [7, 8, 9];
 	//
-	// creates array [1, 2, 3, 4, 5, 6, 7, 8, 9]; num1, num2, num3 are unchanged  
-	// var nums = num1.concat(num2, num3);  
+	// creates array [1, 2, 3, 4, 5, 6, 7, 8, 9]; num1, num2, num3 are unchanged
+	// var nums = num1.concat(num2, num3);
 
-	// var alpha = ['a', 'b', 'c'];  
-	// creates array ["a", "b", "c", 1, 2, 3], leaving alpha unchanged  
+	// var alpha = ['a', 'b', 'c'];
+	// creates array ["a", "b", "c", 1, 2, 3], leaving alpha unchanged
 	// var alphaNumeric = alpha.concat(1, [2, 3]);
-	if (!oper.number())
-	    return false;
-	JsArray* array = new JsArray(mutex());
-	// copy this array
-	for (int i = 0; i < m_length; i++)
-	    array->params().addParam(params().getParam(String(i)));
+
+	ObjList args;
+	extractArgs(this,stack,oper,context,args);
+
+	JsArray* array = new JsArray(context,mutex());
+	// copy this array - only numerically indexed elements!
+	for (int i = 0; i < m_length; i++) {
+	    NamedString* ns = params().getParam(String(i));
+	    ExpOperation* op = YOBJECT(ExpOperation,ns);
+	    op = op ? op->clone() : new ExpOperation(*ns,ns->name(),true);
+	    array->params().addParam(op);
+	}
 	array->setLength(length());
-	// add parameters (JsArray of JsObject)
-	for (int i = (int)oper.number(); i; i--) {
-	    ExpOperation* op = popValue(stack,context);
-	    ExpWrapper* obj = YOBJECT(ExpWrapper,op);
-	    if (!obj)
-		continue;
-	    JsArray* ja = (JsArray*)obj->getObject(YATOM("JsArray"));
+	// add parameters - either basic types or elements of Array
+	while (ExpOperation* op = static_cast<ExpOperation*>(args.remove(false))) {
+	    JsArray* ja = YOBJECT(JsArray,op);
 	    if (ja) {
-		for (int j = 0; j < ja->length(); j++)
-		    array->params().addParam(String(j + array->length()),ja->params().getValue(String(j)));
-		array->setLength(array->length() + ja->length());
+		int len = ja->length();
+		for (int i = 0; i < len; i++) {
+		    NamedString* ns = ja->params().getParam(String(i));
+		    ExpOperation* arg = YOBJECT(ExpOperation,ns);
+		    arg = arg ? arg->clone() : new ExpOperation(*ns,0,true);
+		    const_cast<String&>(arg->name()) = (unsigned int)array->m_length++;
+		    array->params().addParam(arg);
+		}
+		TelEngine::destruct(op);
 	    }
 	    else {
-		JsObject* jo = (JsObject*)obj->getObject(YATOM("JsObject"));
-		if (jo) {
-		    array->params().addParam(new NamedPointer(String(array->length()),jo));
-		    array->setLength(array->length() + 1);
-		    jo->ref();
-		}
-		else
-		    continue;
+		const_cast<String&>(op->name()) = (unsigned int)array->m_length++;
+		array->params().addParam(op);
 	    }
-	    TelEngine::destruct(op);
 	}
-	ExpEvaluator::pushOne(stack,new ExpWrapper(array,0));
+	ExpEvaluator::pushOne(stack,new ExpWrapper(array));
     }
     else if (oper.name() == YSTRING("join")) {
 	// Joins all elements of an array into a string
@@ -647,14 +710,22 @@ bool JsArray::runNative(ObjList& stack, const ExpOperation& oper, GenObject* con
 	// Reverses the order of the elements of an array -- the first becomes the last, and the last becomes the first.
 	// var myArray = ["one", "two", "three"];
 	// myArray.reverse(); => three, two, one
-	NamedList reversed("");
-	String separator = ",";
-	String toCopy;
-	for (int32_t i = 0; i < length(); i++)
-	    toCopy.append(params()[String(i)],separator);
-	reversed.copyParams(params(),toCopy);
-	for (int32_t i = length(); i; i--)
-	    params().setParam(String(length() - i),reversed.getValue(String(i - 1)));
+	if (oper.number())
+	    return false;
+	int i1 = 0;
+	int i2 = length() - 1;
+	for (; i1 < i2; i1++, i2--) {
+	    String s1(i1);
+	    String s2(i2);
+	    NamedString* n1 = params().getParam(s1);
+	    NamedString* n2 = params().getParam(s2);
+	    if (n1)
+		const_cast<String&>(n1->name()) = s2;
+	    if (n2)
+		const_cast<String&>(n2->name()) = s1;
+	}
+	ref();
+	ExpEvaluator::pushOne(stack,new ExpWrapper(this));
     }
     else if (oper.name() == YSTRING("shift")) {
 	// Removes the first element from an array and returns that element
@@ -668,16 +739,30 @@ bool JsArray::runNative(ObjList& stack, const ExpOperation& oper, GenObject* con
 	// myFish before: angel,clown,mandarin,surgeon
 	// myFish after: clown,mandarin,surgeon
 	// Removed this element: angel
-	if (!length())
-	    ExpEvaluator::pushOne(stack,new ExpWrapper(0,0));
-	else {
-	    ExpEvaluator::pushOne(stack,new ExpOperation(params().getValue("0")));
+	if (oper.number())
+	    return false;
+	ObjList* l = params().paramList()->find("0");
+	if (l) {
+	    NamedString* ns = static_cast<NamedString*>(l->get());
+	    params().paramList()->remove(ns,false);
+	    ExpOperation* op = YOBJECT(ExpOperation,ns);
+	    if (!op) {
+		op = new ExpOperation(*ns,0,true);
+		TelEngine::destruct(ns);
+	    }
+	    ExpEvaluator::pushOne(stack,op);
 	    // shift : value n+1 becomes value n
-	    for (int32_t i = 0; i < length() - 1; i++)
-		params().setParam(String(i),params().getValue(String(i + 1)));
-	    params().clearParam(String(length() - 1));
-	    setLength(length() - 1);
+	    for (int32_t i = 0; ; i++) {
+		ns = static_cast<NamedString*>((*params().paramList())[String(i + 1)]);
+		if (!ns) {
+		    setLength(i);
+		    break;
+		}
+		const_cast<String&>(ns->name()) = i;
+	    }
 	}
+	else
+	    ExpEvaluator::pushOne(stack,new ExpWrapper(0,0));
     }
     else if (oper.name() == YSTRING("unshift")) {
 	// Adds one or more elements to the front of an array and returns the new length of the array
@@ -692,20 +777,20 @@ bool JsArray::runNative(ObjList& stack, const ExpOperation& oper, GenObject* con
 	// New length: 4
 	// shift array
 	int32_t shift = (int32_t)oper.number();
-	for (int32_t i = length(); i; i--)
-	    params().setParam(String(i - 1 + shift),params().getValue(String(i - 1)));
-	for (int32_t i = shift; i; i--) {
+	for (int32_t i = length() + shift - 1; i >= shift; i--) {
+	    NamedString* ns = static_cast<NamedString*>((*params().paramList())[String(i - shift)]);
+	    if (ns) {
+		String index(i);
+		params().clearParam(index);
+		const_cast<String&>(ns->name()) = index;
+	    }
+	}
+	for (int32_t i = shift - 1; i >= 0; i--) {
 	    ExpOperation* op = popValue(stack,context);
-	    ExpWrapper* obj = YOBJECT(ExpWrapper,op);
-	    if (!obj)
+	    if (!op)
 		continue;
-	    JsObject* jo = (JsObject*)obj->getObject(YATOM("JsObject"));
-	    if (!jo)
-		continue;
-	    jo->ref();
-	    params().clearParam(String(i - 1));
-	    params().setParam(new NamedPointer(String(i - 1),jo));
-	    TelEngine::destruct(op);
+	    const_cast<String&>(op->name()) = i;
+	    params().paramList()->insert(op);
 	}
 	setLength(length() + shift);
 	ExpEvaluator::pushOne(stack,new ExpOperation((int64_t)length()));
@@ -760,7 +845,7 @@ bool JsArray::runNative(ObjList& stack, const ExpOperation& oper, GenObject* con
 	    }
 	}
 	TelEngine::destruct(op1);
-	ExpEvaluator::pushOne(stack,new ExpOperation(String(index)));
+	ExpEvaluator::pushOne(stack,new ExpOperation((int64_t)index));
 	return true;
     }
     else
@@ -771,92 +856,134 @@ bool JsArray::runNative(ObjList& stack, const ExpOperation& oper, GenObject* con
 bool JsArray::runNativeSlice(ObjList& stack, const ExpOperation& oper, GenObject* context)
 {
     // Extracts a section of an array and returns a new array.
-    // var myHonda = { color: "red", wheels: 4, engine: { cylinders: 4, size: 2.2 } };  
-    // var myCar = [myHonda, 2, "cherry condition", "purchased 1997"];  
-    // var newCar = myCar.slice(0, 2); 
-    if (!oper.number())
-	return false;
-    // begin | end > 0 offset from the start of the array
-    //	       < 0 offset from the end of the array
-    // end missing -> go to end of array  
-    int begin = length(), end = length();
-    for (int i = (int)oper.number(); i; i--) {
-	ExpOperation* op = popValue(stack,context);
-	if (op->isInteger()) {
-	    end = begin;
-	    begin = (int)op->number();
-	}
-	TelEngine::destruct(op);
+    // var myHonda = { color: "red", wheels: 4, engine: { cylinders: 4, size: 2.2 } };
+    // var myCar = [myHonda, 2, "cherry condition", "purchased 1997"];
+    // var newCar = myCar.slice(0, 2);
+    int32_t begin = 0, end = length();
+    switch (oper.number()) {
+	case 2:
+	    {   // get end of interval
+		ExpOperation* op = popValue(stack,context);
+		if (op && op->isInteger())
+		    end = op->number();
+		TelEngine::destruct(op);
+	    }
+	// intentional fallthrough
+	case 1:
+	    {
+		ExpOperation* op = popValue(stack,context);
+		if (op && op->isInteger())
+		    begin = op->number();
+		TelEngine::destruct(op);
+	    }
+	    break;
+	case 0:
+	    break;
+	default:
+	    // maybe we should ignore the rest of the given parameters?
+	    return false;
     }
-    if (begin < 0)
+    if (begin < 0) {
 	begin = length() + begin;
+	if (begin < 0)
+	    begin = 0;
+    }
     if (end < 0)
 	end = length() + end;
-    if (end < begin)
-	return false;
-    // TODO
-    //JsArray* slice = new JsArray(mutex());
-    for (long int i = begin; i < end; i++) {
-//	slice->params().addParam(new NamedString(String(i - begin),
+ 
+    JsArray* array = new JsArray(context,mutex());
+    for (int32_t i = begin; i < end; i++) {
+	NamedString* ns = params().getParam(String(i));
+	if (!ns) {
+	    // if missing, insert undefined element in array also
+	    array->m_length++;
+	    continue;
+	}
+	ExpOperation* arg = YOBJECT(ExpOperation,ns);
+	arg = arg ? arg->clone() : new ExpOperation(*ns,0,true);
+	const_cast<String&>(arg->name()) = (unsigned int)array->m_length++;
+	array->params().addParam(arg);
     }
+    ExpEvaluator::pushOne(stack,new ExpWrapper(array));
     return true;
 }
 
 bool JsArray::runNativeSplice(ObjList& stack, const ExpOperation& oper, GenObject* context)
 {
-    // Changes the content of an array, adding new elements while removing old elements. 
+    // Changes the content of an array, adding new elements while removing old elements.
     // Returns an array containing the removed elements
     // array.splice(index , howMany[, element1[, ...[, elementN]]])
     // array.splice(index ,[ howMany[, element1[, ...[, elementN]]]])
-    ObjList arguments;
-    int argc = extractArgs(this,stack,oper,context,arguments);
+    ObjList args;
+    int argc = extractArgs(this,stack,oper,context,args);
     if (!argc)
 	return false;
     // get start index
-    ExpOperation* op = static_cast<ExpOperation*>(arguments[0]);
-    int begin = (int)op->number();
+    int32_t len = length();
+    ExpOperation* op = static_cast<ExpOperation*>(args.remove(false));
+    int32_t begin = (int)(op->number() > len ? len : op->number());
     if (begin < 0)
-	begin = length() + begin;
-    // get count to delete
-    int count = length() - begin;
-    if (arguments.count() > 1) {
-	// get count
-	op = static_cast<ExpOperation*>(arguments[1]);
-	count = (int)op->number();
+	begin = len + begin > 0 ? len + begin : 0;
+    TelEngine::destruct(op);
+    argc--;
+    // get count of objects to delete
+    int32_t delCount = len - begin;
+    if (argc) {
+	op = static_cast<ExpOperation*>(args.remove(false));
+	// howMany is negative, set it to 0
+	if (op->number() < 0)
+	    delCount = 0;
+	// if howMany is greater than the length of remaining elements from start index, do not set it
+	else if (op->number() < delCount)
+	    delCount = op->number();
+	TelEngine::destruct(op);
+	argc--;
     }
 
     // remove elements
-    JsArray* removed = new JsArray(mutex());
-    for (int i = begin; i < begin + count; i++) {
-	removed->params().setParam(String(begin + count - begin),params().getValue(String(i)));
-	params().clearParam(String(i));
+    JsArray* removed = new JsArray(context,mutex());
+    for (int32_t i = begin; i < begin + delCount; i++) {
+	NamedString* ns = params().getParam(String(i));
+	if (!ns) {
+	    // if missing, insert undefined element in array also
+	    removed->m_length++;
+	    continue;
+	}
+	params().paramList()->remove(ns,false);
+	ExpOperation* op = YOBJECT(ExpOperation,ns);
+	if (!op) {
+	    op = new ExpOperation(*ns,0,true);
+	    TelEngine::destruct(ns);
+	}
+	const_cast<String&>(op->name()) = (unsigned int)removed->m_length++;
+	removed->params().addParam(op);
     }
-    removed->setLength(count);
 
-    // add the trailing array elements
-    // index for trailing array elements arfer removing the specified count
-    int shiftIdx = begin + count;
-    // shift how many positions for trailing array elements
-    int shiftWith = arguments.count() > 2 ? arguments.count() - 2 - count : -count;
-    if (shiftWith > 0) {
-	// shift everything starting at index shiftIdx with shiftWith positions to the right
-	for (int i = length(); i > shiftIdx; i--)
-	    params().setParam(String(i - 1 + shiftWith),params().getValue(String(i - 1)));
+    int32_t shiftIdx = argc - delCount;
+    // shift elements to make room for those that are to be inserted or move the ones that remained
+    // after delete
+    if (shiftIdx > 0) {
+	for (int32_t i = m_length - 1; i >= begin + delCount; i--) {
+	    NamedString* ns = static_cast<NamedString*>((*params().paramList())[String(i)]);
+	    if (ns)
+		const_cast<String&>(ns->name()) = i + shiftIdx;
+	}
     }
-    else if (shiftWith < 0) {
-	// shift everything starting at index shiftIdx with shiftWith positions to the left
-	for (int i = shiftIdx; i < length(); i++)
-	    params().setParam(String(i + shiftWith),params().getValue(String(i)));
+    else if (shiftIdx < 0) {
+	for (int32_t i = begin + delCount; i < m_length; i++) {
+	    NamedString* ns = static_cast<NamedString*>((*params().paramList())[String(i)]);
+	    if (ns)
+		const_cast<String&>(ns->name()) = i + shiftIdx;
+	}
     }
-    // insert the new ones 
-    for (int i = begin;(arguments.count() > 2) && (i < length()); i++) {
-	GenObject* obj = arguments[2 + i - begin];
-	params().setParam(new NamedPointer(String(i),obj));
+    setLength(length() + shiftIdx);
+    // insert the new elements
+    for (int i = 0; i < argc; i++) {
+	ExpOperation* arg = static_cast<ExpOperation*>(args.remove(false));
+	const_cast<String&>(arg->name()) = (unsigned int)(begin + i);
+	params().addParam(arg);
     }
-    // set length
-    setLength(arguments.count() > 2 ? length() + arguments.count() - 2 - count : length() - count);
-    // push the removed array on stack
-    ExpEvaluator::pushOne(stack,new ExpWrapper(removed,0));
+    ExpEvaluator::pushOne(stack,new ExpWrapper(removed));
     return true;
 }
 
@@ -864,15 +991,18 @@ class JsComparator
 {
 public:
     JsComparator(const char* funcName, ScriptRun* runner)
-	: m_name(funcName), m_runner(runner)
+	: m_name(funcName), m_runner(runner), m_failed(false)
 	{ }
     const char* m_name;
     ScriptRun* m_runner;
+    bool m_failed;
 };
 
 int compare(GenObject* op1, GenObject* op2, void* data)
 {
     JsComparator* cmp = static_cast<JsComparator*>(data);
+    if (cmp && cmp->m_failed)
+	return 0;
     if (!(cmp && cmp->m_runner))
 	return ::strcmp(*(static_cast<String*>(op1)),*(static_cast<String*>(op2)));
     ScriptRun* runner = cmp->m_runner->code()->createRunner(cmp->m_runner->context());
@@ -884,10 +1014,16 @@ int compare(GenObject* op1, GenObject* op2, void* data)
     ScriptRun::Status rval = runner->call(cmp->m_name,stack);
     int ret = 0;
     if (ScriptRun::Succeeded == rval) {
-	String* sret = static_cast<String*>(ExpEvaluator::popOne(runner->stack()));
-	ret = sret->toInteger();
-	TelEngine::destruct(sret);
+	ExpOperation* sret = static_cast<ExpOperation*>(ExpEvaluator::popOne(runner->stack()));
+	if (sret) {
+	    ret = sret->toInteger();
+	    TelEngine::destruct(sret);
+	}
+	else
+	    cmp->m_failed = true;
     }
+    else
+	cmp->m_failed = true;
     TelEngine::destruct(runner);
     return ret;
 }
@@ -898,29 +1034,38 @@ bool JsArray::runNativeSort(ObjList& stack, const ExpOperation& oper, GenObject*
     ExpOperation* op = 0;
     if (extractArgs(this,stack,oper,context,arguments))
 	op = static_cast<ExpOperation*>(arguments[0]);
-    ObjList sorted;
-    // Copy the arguments in a ObjList for sorting
-    for (int i = 0;i < length();i++) {
-	NamedString* param = params().getParam(String(i));
-	sorted.append(param);
-	params().clearParam(param,false);
-    }
     ScriptRun* runner = YOBJECT(ScriptRun,context);
     if (op && !runner)
 	return false;
+    ObjList sorted;
+    ObjList* last = &sorted;
+    // Copy the arguments in a ObjList for sorting
+    for (ObjList* o = params().paramList()->skipNull(); o; o = o->skipNext()) {
+	NamedString* str = static_cast<NamedString*>(o->get());
+	if (str->name().toInteger(-1) > -1)
+	    (last = last->append(str))->setDelete(false);
+    }
     JsComparator* comp = op ? new JsComparator(op->name() ,runner) : 0;
     sorted.sort(&compare,comp);
+    bool ok = comp ? !comp->m_failed : true;
     delete comp;
-    int i = 0;
-    for (ObjList* o = sorted.skipNull();o;o = o->skipNext()) {
-	NamedString* slice = static_cast<NamedString*>(o->get());
-	String* name = (String*)(&slice->name());
-	*name = String(i++);
-	params().addParam(slice);
-	o->setDelete(false);
+    if (ok) {
+	for (ObjList* o = params().paramList()->skipNull(); o;) {
+	    NamedString* str = static_cast<NamedString*>(o->get());
+	    if (str && str->name().toInteger(-1) > -1)
+		o->remove(false);
+	    else
+		o = o->skipNext();
+	}
+	int i = 0;
+	last = params().paramList()->last();
+	for (ObjList* o = sorted.skipNull();o; o = o->skipNull()) {
+	    ExpOperation* slice = static_cast<ExpOperation*>(o->remove(false));
+	    const_cast<String&>(slice->name()) = i++;
+	    last = last->append(slice);
+	}
     }
-    setLength(i);
-    return true;
+    return ok;
 }
 
 
@@ -939,6 +1084,15 @@ JsRegExp::JsRegExp(Mutex* mtx, const char* name, const char* rexp, bool insensit
     params().addParam("basicPosix",String::boolText(!extended));
 }
 
+JsRegExp::JsRegExp(Mutex* mtx, const Regexp& rexp, bool frozen)
+    : JsObject("RegExp",mtx),
+      m_regexp(rexp)
+{
+    params().addParam(new ExpFunction("test"));
+    params().addParam("ignoreCase",String::boolText(rexp.isCaseInsensitive()));
+    params().addParam("basicPosix",String::boolText(!rexp.isExtended()));
+}
+
 bool JsRegExp::runNative(ObjList& stack, const ExpOperation& oper, GenObject* context)
 {
     XDebug(DebugAll,"JsRegExp::runNative() '%s' in '%s' [%p]",
@@ -956,6 +1110,47 @@ bool JsRegExp::runNative(ObjList& stack, const ExpOperation& oper, GenObject* co
     return true;
 }
 
+JsObject* JsRegExp::runConstructor(ObjList& stack, const ExpOperation& oper, GenObject* context)
+{
+    ObjList args;
+    switch (extractArgs(stack,oper,context,args)) {
+	case 1:
+	case 2:
+	    break;
+	default:
+	    return 0;
+    }
+    ExpOperation* pattern = static_cast<ExpOperation*>(args[0]);
+    ExpOperation* flags = static_cast<ExpOperation*>(args[1]);
+    if (!pattern)
+	return 0;
+    bool insensitive = false;
+    bool extended = true;
+    if (flags)  {
+	const char* f = *flags;
+	char c = *f++;
+	while (c) {
+	    switch (c) {
+		case 'i':
+		    c = *f++;
+		    insensitive = true;
+		    break;
+		case 'b':
+		    c = *f++;
+		    extended = false;
+		    break;
+		default:
+		    c = 0;
+	    }
+	}
+    }
+    if (!ref())
+	return 0;
+    JsRegExp* obj = new JsRegExp(mutex(),*pattern,*pattern,insensitive,extended);
+    obj->params().addParam(new ExpWrapper(this,protoName()));
+    return obj;
+}
+
 
 bool JsMath::runNative(ObjList& stack, const ExpOperation& oper, GenObject* context)
 {
