@@ -1,10 +1,10 @@
-/*
+/**
  * yatescript.h
  * Yet Another (Java)script library
  * This file is part of the YATE Project http://YATE.null.ro
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2011-2013 Null Team
+ * Copyright (C) 2011-2014 Null Team
  *
  * This software is distributed under multiple licenses;
  * see the COPYING file in the main directory for licensing
@@ -948,19 +948,21 @@ public:
      */
     inline ExpOperation(const ExpOperation& original)
 	: NamedString(original.name(),original),
-	  m_opcode(original.opcode()), m_number(original.number()),
-	  m_lineNo(0), m_barrier(original.barrier())
+	  m_opcode(original.opcode()), m_number(original.number()), m_bool(original.isBoolean()),
+	  m_isNumber(original.isNumber()), m_lineNo(0), m_barrier(original.barrier())
 	{ }
 
     /**
      * Copy constructor with renaming, to be used for named results
      * @param original Operation to copy
      * @param name Name of the newly created operation
+     * @param copyType True to copy operation type, false to create an OpcPush
      */
-    inline ExpOperation(const ExpOperation& original, const char* name)
+    inline ExpOperation(const ExpOperation& original, const char* name, bool copyType = true)
 	: NamedString(name,original),
-	  m_opcode(original.opcode()), m_number(original.number()),
-	  m_lineNo(0), m_barrier(original.barrier())
+	  m_opcode(copyType ? original.opcode() : ExpEvaluator::OpcPush),
+	  m_number(original.number()), m_bool(original.isBoolean()),
+	  m_isNumber(original.isNumber()), m_lineNo(0), m_barrier(original.barrier())
 	{ }
 
     /**
@@ -973,8 +975,10 @@ public:
 	: NamedString(name,value),
 	  m_opcode(ExpEvaluator::OpcPush),
 	  m_number(autoNum ? value.toInt64(nonInteger()) : nonInteger()),
+	  m_bool(autoNum && value.isBoolean()),
+	  m_isNumber(autoNum && (value == YSTRING("NaN") || m_number != nonInteger())),
 	  m_lineNo(0), m_barrier(false)
-	{ if (autoNum && value.isBoolean()) m_number = value.toBoolean() ? 1 : 0; }
+	{ if (m_bool) { m_number = value.toBoolean() ? 1 : 0; m_isNumber = true;} }
 
     /**
      * Push literal string constructor
@@ -983,7 +987,8 @@ public:
      */
     inline explicit ExpOperation(const char* value, const char* name = 0)
 	: NamedString(name,value),
-	  m_opcode(ExpEvaluator::OpcPush), m_number(nonInteger()), m_lineNo(0), m_barrier(false)
+	  m_opcode(ExpEvaluator::OpcPush), m_number(nonInteger()), m_bool(false),
+	  m_isNumber(false), m_lineNo(0), m_barrier(false)
 	{ }
 
     /**
@@ -994,7 +999,7 @@ public:
     inline explicit ExpOperation(int64_t value, const char* name = 0)
 	: NamedString(name,"NaN"),
 	  m_opcode(ExpEvaluator::OpcPush),
-	  m_number(value), m_lineNo(0), m_barrier(false)
+	  m_number(value), m_bool(false), m_isNumber(true), m_lineNo(0), m_barrier(false)
 	{ if (value != nonInteger()) String::operator=(value); }
 
     /**
@@ -1005,7 +1010,8 @@ public:
     inline explicit ExpOperation(bool value, const char* name = 0)
 	: NamedString(name,String::boolText(value)),
 	  m_opcode(ExpEvaluator::OpcPush),
-	  m_number(value ? 1 : 0), m_lineNo(0), m_barrier(false)
+	  m_number(value ? 1 : 0), m_bool(true), m_isNumber(true),
+	  m_lineNo(0), m_barrier(false)
 	{ }
 
     /**
@@ -1017,7 +1023,8 @@ public:
      */
     inline ExpOperation(ExpEvaluator::Opcode oper, const char* name = 0, int64_t value = nonInteger(), bool barrier = false)
 	: NamedString(name,""),
-	  m_opcode(oper), m_number(value), m_lineNo(0), m_barrier(barrier)
+	  m_opcode(oper), m_number(value), m_bool(false), m_isNumber(false),
+	  m_lineNo(0), m_barrier(barrier)
 	{ }
 
     /**
@@ -1029,7 +1036,8 @@ public:
      */
     inline ExpOperation(ExpEvaluator::Opcode oper, const char* name, const char* value, bool barrier = false)
 	: NamedString(name,value),
-	  m_opcode(oper), m_number(nonInteger()), m_lineNo(0), m_barrier(barrier)
+	  m_opcode(oper), m_number(nonInteger()), m_bool(false), m_isNumber(false),
+	  m_lineNo(0), m_barrier(barrier)
 	{ }
 
     /**
@@ -1042,7 +1050,8 @@ public:
      */
     inline ExpOperation(ExpEvaluator::Opcode oper, const char* name, const char* value, int64_t number, bool barrier)
 	: NamedString(name,value),
-	  m_opcode(oper), m_number(number), m_lineNo(0), m_barrier(barrier)
+	  m_opcode(oper), m_number(number), m_bool(false), m_isNumber(true),
+	  m_lineNo(0), m_barrier(barrier)
 	{ }
 
     /**
@@ -1066,6 +1075,20 @@ public:
     inline int64_t number() const
 	{ return m_number; }
 
+    /**
+     * Check if a boolean value is stored
+     * @return True if a boolean value is stored
+     */
+    inline bool isBoolean() const
+	{ return m_bool; }
+
+    /**
+     * Check if a number type value is stored
+     * @return True if a number type value is stored
+     */
+    inline bool isNumber() const
+	{ return m_isNumber; }
+
     /**
      * Check if this operation acts as an evaluator barrier on the stack
      * @return True if an expression should not pop this operation off the stack
@@ -1093,7 +1116,7 @@ public:
      * @return Assigned number
      */
     inline int64_t operator=(int64_t num)
-	{ m_number = num; String::operator=(num); return num; }
+	{ m_number = num; String::operator=(num); m_isNumber = true; return num; }
 
     /**
      * Retrieve the numeric value of the operation
@@ -1101,6 +1124,12 @@ public:
      */
     virtual int64_t valInteger() const;
 
+    /**
+     * Convert to number
+     * @return Value converted to number, NaN if not possible of if stored value is NaN
+     */
+    virtual int64_t toNumber() const;
+
     /**
      * Retrieve the boolean value of the operation
      * @return True if the operation is to be interpreted as true value
@@ -1138,6 +1167,8 @@ public:
 private:
     ExpEvaluator::Opcode m_opcode;
     int64_t m_number;
+    bool m_bool;
+    bool m_isNumber;
     unsigned int m_lineNo;
     bool m_barrier;
 };
@@ -1854,6 +1885,14 @@ public:
      */
     JsObject(Mutex* mtx, const char* name, bool frozen = false);
 
+    /**
+     * Constructor for an empty object with prototype
+     * @param context Script context from which Object prototype is obtainend
+     * @param mtx Pointer to the mutex that serializes this object
+     * @param frozen True if the object is to be frozen from creation
+     */
+    JsObject(GenObject* context, Mutex* mtx = 0, bool frozen = false);
+
     /**
      * Destructor
      */
@@ -1881,6 +1920,13 @@ public:
     inline JsObject* clone() const
 	{ return clone(toString()); }
 
+    /**
+     * Set the object prototype
+     * @param context  Pointer to arbitrary object passed from evaluation methods
+     * @param objName Name of the object prototype to set the this object
+     */
+    void setPrototype(GenObject* context, const String& objName);
+
     /**
      * Deep copy method
      * @param mtx Pointer to the mutex that serializes the copied object
@@ -2187,13 +2233,16 @@ private:
  */
 class YSCRIPT_API JsArray : public JsObject
 {
+    friend class JsObject;
     YCLASS(JsArray,JsObject)
 public:
+
     /**
-     * Constructor
+     * Constructor for an empty array with prototype
+     * @param context Script context from which Array prototype is obtainend
      * @param mtx Pointer to the mutex that serializes this object
      */
-    JsArray(Mutex* mtx = 0);
+    JsArray(GenObject* context, Mutex* mtx = 0);
 
     /**
      * Constructor for an empty array
@@ -2251,7 +2300,23 @@ public:
      */
     virtual bool runField(ObjList& stack, const ExpOperation& oper, GenObject* context);
 
+    /**
+     * Array constructor initialization
+     * @param construct The Array function
+     */
+    virtual void initConstructor(JsFunction* construct);
+
+    /**
+     * Array object constructor, it's run on the prototype
+     * @param stack Evaluation stack in use
+     * @param oper Constructor function to evaluate
+     * @param context Pointer to arbitrary object passed from evaluation methods
+     * @return New created and populated Javascript Array object
+     */
+    virtual JsObject* runConstructor(ObjList& stack, const ExpOperation& oper, GenObject* context);
+
 protected:
+
     /**
      * Clone and rename method
      * @param name Name of the cloned object
@@ -2271,6 +2336,13 @@ protected:
     bool runNative(ObjList& stack, const ExpOperation& oper, GenObject* context);
 
 private:
+
+    /**
+     * Private constructor
+     * @param mtx Pointer to the mutex that serializes this object
+     */
+    JsArray(Mutex* mtx = 0);
+
     bool runNativeSlice(ObjList& stack, const ExpOperation& oper, GenObject* context);
     bool runNativeSplice(ObjList& stack, const ExpOperation& oper, GenObject* context);
     bool runNativeSort(ObjList& stack, const ExpOperation& oper, GenObject* context);
@@ -2303,6 +2375,14 @@ public:
     JsRegExp(Mutex* mtx, const char* name, const char* rexp = 0, bool insensitive = false,
 	bool extended = true, bool frozen = false);
 
+    /**
+     * Constructor from existing Regexp
+     * @param mtx Pointer to the mutex that serializes this object
+     * @param rexp Regular expression to copy
+     * @param frozen True to create an initially frozen object
+     */
+    JsRegExp(Mutex* mtx, const Regexp& rexp, bool frozen = false);
+
     /**
      * Access the internal Regexp object that does the matching
      * @return Const reference to the internal Regexp object
@@ -2317,6 +2397,15 @@ public:
     inline Regexp& regexp()
 	{ return m_regexp; }
 
+    /**
+     * RegExp object constructor, it's run on the prototype
+     * @param stack Evaluation stack in use
+     * @param oper Constructor function to evaluate
+     * @param context Pointer to arbitrary object passed from evaluation methods
+     * @return New created and populated Javascript RegExp object
+     */
+    virtual JsObject* runConstructor(ObjList& stack, const ExpOperation& oper, GenObject* context);
+
 protected:
     /**
      * Clone and rename method
@@ -2402,8 +2491,9 @@ public:
     /**
      * Adjust a file script path to include default if needed
      * @param script File path to adjust
+     * @param extraInc True to check the extra include path first
      */
-    void adjustPath(String& script) const;
+    void adjustPath(String& script, bool extraInc = false) const;
 
     /**
      * Retrieve the base script path
@@ -2412,12 +2502,20 @@ public:
     inline const String& basePath() const
 	{ return m_basePath; }
 
+    /**
+     * Retrieve the extra include script path
+     * @return Include path added to relative script paths
+     */
+    inline const String& includePath() const
+	{ return m_includePath; }
+
     /**
      * Set the base script path
      * @param path Base path to add to relative script paths
+     * @param incPath Extra include path to add to relative script paths
      */
-    inline void basePath(const char* path)
-	{ m_basePath = path; }
+    inline void basePath(const char* path, const char* incPath = 0)
+	{ m_basePath = path; m_includePath = incPath; }
 
     /**
      * Retrieve the last parsed file name
@@ -2437,10 +2535,11 @@ public:
      * Check if the script or any includes have changed
      * @param file Name of the file to check
      * @param path New base path to check
+     * @param incPath New extra include path to check
      * @return True if the script may have changed, false if not changed
      */
-    inline bool scriptChanged(const char* file, const String& path) const
-	{ return (path != m_basePath) || scriptChanged(file); }
+    inline bool scriptChanged(const char* file, const String& path, const String& incPath = String::empty()) const
+	{ return (path != m_basePath) || (incPath != m_includePath) || scriptChanged(file); }
 
     /**
      * Set whether the Javascript code should be linked or not
@@ -2468,9 +2567,10 @@ public:
     /**
      * Parse a complete block of JSON text
      * @param text JSON text to parse
-     * @return JsObject holding the content of JSON, must be dereferenced after use, NULL if parse error
+     * @param mtx Pointer to the mutex that serializes this object
+     * @return ExpOperation holding the content of JSON, must be dereferenced after use, NULL if parse error
      */
-    static JsObject* parseJSON(const char* text);
+    static ExpOperation* parseJSON(const char* text, Mutex* mtx = 0);
 
     /**
      * Get a "null" object wrapper that will identity match another "null"
@@ -2479,6 +2579,12 @@ public:
      */
     static ExpOperation* nullClone(const char* name = 0);
 
+    /**
+     * Obtain the "null" object
+     * @return Referenced "null" object (0 if ref() fails)
+     */
+    static JsObject* nullObject();
+
     /**
      * Check if an operation holds a null value
      * @return True if the operation holds a null object
@@ -2493,6 +2599,7 @@ public:
 
 private:
     String m_basePath;
+    String m_includePath;
     String m_parsedFile;
     bool m_allowLink;
     bool m_allowTrace;
