@@ -5,7 +5,7 @@
  * SDP media handling
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2013 Null Team
+ * Copyright (C) 2004-2014 Null Team
  *
  * This software is distributed under multiple licenses;
  * see the COPYING file in the main directory for licensing
@@ -29,7 +29,7 @@ namespace TelEngine {
 SDPSession::SDPSession(SDPParser* parser)
     : m_parser(parser), m_mediaStatus(MediaMissing),
       m_rtpForward(false), m_sdpForward(false), m_rtpMedia(0),
-      m_sdpSession(0), m_sdpVersion(0),
+      m_sdpSession(0), m_sdpVersion(0), m_sdpHash(YSTRING_INIT_HASH),
       m_secure(m_parser->m_secure), m_rfc2833(m_parser->m_rfc2833),
       m_ipv6(false), m_enabler(0), m_ptr(0)
 {
@@ -39,7 +39,7 @@ SDPSession::SDPSession(SDPParser* parser)
 SDPSession::SDPSession(SDPParser* parser, NamedList& params)
     : m_parser(parser), m_mediaStatus(MediaMissing),
       m_rtpForward(false), m_sdpForward(false), m_rtpMedia(0),
-      m_sdpSession(0), m_sdpVersion(0),
+      m_sdpSession(0), m_sdpVersion(0), m_sdpHash(YSTRING_INIT_HASH),
       m_ipv6(false), m_enabler(0), m_ptr(0)
 {
     setSdpDebug();
@@ -148,7 +148,7 @@ bool SDPSession::dispatchRtp(SDPMedia* media, const char* addr, bool start,
 	const String* suite = m->getParam("ocrypto_suite");
 	const String* key = m->getParam("ocrypto_key");
 	const String* params = m->getParam("ocrypto_params");
-	if (suite && key && (tag > 0)) {
+	if (suite && key && (tag >= 0)) {
 	    String sdes(tag);
 	    sdes << " " << *suite << " " << *key;
 	    if (params)
@@ -328,9 +328,7 @@ MimeSdpBody* SDPSession::createSDP(const char* addr, ObjList* mediaList)
     // if we got no media descriptors we simply create no SDP
     if (!mediaList)
 	return 0;
-    if (m_sdpSession)
-	++m_sdpVersion;
-    else
+    if (!m_sdpSession)
 	m_sdpVersion = m_sdpSession = Time::secNow();
 
     // override the address with the externally advertised if needed
@@ -340,15 +338,14 @@ MimeSdpBody* SDPSession::createSDP(const char* addr, ObjList* mediaList)
 	m_originAddr = addr ? addr : m_host.safe();
     // no address means on hold or muted
     String origin;
-    origin << "yate " << m_sdpSession << " " << m_sdpVersion;
-    origin << " ";
     int f = addIP(origin,m_originAddr);
     String conn;
     addIP(conn,addr,f);
 
-    MimeSdpBody* sdp = new MimeSdpBody;
+    MimeSdpBody* sdp = new MimeSdpBody(true);
     sdp->addLine("v","0");
-    sdp->addLine("o",origin);
+    // insert incomplete origin just for hashing purpose
+    NamedString* org = sdp->addLine("o",origin);
     sdp->addLine("s",m_parser->m_sessionName);
     sdp->addLine("c",conn);
     sdp->addLine("t","0 0");
@@ -549,6 +546,14 @@ MimeSdpBody* SDPSession::createSDP(const char* addr, ObjList* mediaList)
 		sdp->addLine("a","encryption:optional");
 	}
     }
+    // increment version if body hash changed
+    if ((YSTRING_INIT_HASH != m_sdpHash) && (sdp->hash() != m_sdpHash))
+	m_sdpVersion++;
+    m_sdpHash = sdp->hash();
+    // insert version in the origin line
+    origin.clear();
+    origin << "yate " << m_sdpSession << " " << m_sdpVersion << " " << *org;
+    *org = origin;
 
     return sdp;
 }
@@ -918,7 +923,7 @@ void SDPSession::dispatchingRtp(Message*& msg, SDPMedia* media)
     XDebug(m_enabler,DebugAll,"SDPSession::dispatchingRtp(%p,%p) [%p]",msg,media,m_ptr);
 }
 
-// Set data used in debug 
+// Set data used in debug
 void SDPSession::setSdpDebug(DebugEnabler* enabler, void* ptr)
 {
     m_enabler = enabler ? enabler : static_cast<DebugEnabler*>(m_parser);
