@@ -1,10 +1,10 @@
 /**
  * engine.cpp
  * Yet Another SIP Stack
- * This file is part of the YATE Project http://YATE.null.ro 
+ * This file is part of the YATE Project http://YATE.null.ro
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2013 Null Team
+ * Copyright (C) 2004-2014 Null Team
  *
  * This software is distributed under multiple licenses;
  * see the COPYING file in the main directory for licensing
@@ -134,7 +134,7 @@ void SIPParty::setAddr(const String& addr, int port, bool local)
     int& p = local ? m_localPort : m_partyPort;
     a = addr;
     p = port;
-    DDebug(DebugAll,"SIPParty updated %s address '%s:%d' [%p]",
+    DDebug(DebugAll,"SIPParty updated %s address '%s' [%p]",
 	local ? "local" : "remote",SocketAddr::appendTo(a,p).c_str(),this);
 }
 
@@ -171,13 +171,15 @@ SIPEngine::SIPEngine(const char* userAgent)
     : Mutex(true,"SIPEngine"),
       m_t1(500000), m_t4(5000000), m_reqTransCount(5), m_rspTransCount(6),
       m_maxForwards(70),
-      m_cseq(0), m_flags(0), m_lazyTrying(false),
+      m_flags(0), m_lazyTrying(false),
       m_userAgent(userAgent), m_nc(0), m_nonce_time(0),
       m_nonce_mutex(false,"SIPEngine::nonce"),
       m_autoChangeParty(false)
 {
     debugName("sipengine");
     DDebug(this,DebugInfo,"SIPEngine::SIPEngine() [%p]",this);
+    m_seq = new SIPSequence;
+    m_seq->deref();
     if (m_userAgent.null())
 	m_userAgent << "YATE/" << YATE_VERSION;
     m_allowed = "ACK";
