@@ -5,7 +5,7 @@
  * Cdr builder
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2013 Null Team
+ * Copyright (C) 2004-2014 Null Team
  *
  * This software is distributed under multiple licenses;
  * see the COPYING file in the main directory for licensing
@@ -355,7 +355,7 @@ void CdrBuilder::emit(const char *operation)
 
     if (String(operation) == YSTRING("update") && !s_cdrUpdates)
 	return;
-    
+
     m_first = false;
 
     DDebug("cdrbuild",DebugAll,"Emit '%s' for '%s' status '%s'",
@@ -391,7 +391,7 @@ void CdrBuilder::emit(const char *operation)
 	m_durationTime.getTime(tmp,t_hangup - m_start);
 	m->addParam("call_duration",tmp);
     }
-    
+
     if (!getValue("external")) {
 	const char* ext = 0;
 	if (m_dir == YSTRING("incoming"))
@@ -695,7 +695,6 @@ void CustomTimer::process(const String& value)
 	extractUsec(value);
 	return;
     }
-    Debug(DebugWarn,"Requested to process: %s",value.c_str());
     String tmp = value;
     int count = 4;
     // YYYY or YY the year
@@ -705,11 +704,9 @@ void CustomTimer::process(const String& value)
 	if (i >= 0)
 	    count = 2;
     }
-    if (i >= 0) {
-	Debug(DebugNote," i = %d count = %d",i,count);
+    if (i >= 0)
 	tmp = tmp.substr(0,i) + ((count == 2) ? YSTRING("%y") : YSTRING("%Y")) +
 		tmp.substr(i + count);
-    }
     // MM month
     i = tmp.find(YSTRING("MM"));
     if (i >= 0)
