@@ -5,7 +5,7 @@
  * External module handler
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2013 Null Team
+ * Copyright (C) 2004-2014 Null Team
  * Portions copyright (C) 2005 Maciek Kaminski
  *
  * This software is distributed under multiple licenses;
@@ -72,6 +72,9 @@ static const char* s_cmds[] = {
     0
 };
 
+static const char s_helpExternalCmd[] = "external [info] [stop scriptname] [[start|restart] scriptname [parameter]] [execute progname [parameter]]";
+static const char s_helpExternalInfo[] = "List, (re)start and stop scripts or execute an external program";
+
 class ExtModReceiver;
 class ExtModChan;
 
@@ -248,6 +251,7 @@ private:
     void closeIn();
     void closeOut();
     void closeAudio();
+    bool outputLineInternal(const char* line, int len);
     int m_role;
     bool m_dead;
     int m_use;
@@ -261,6 +265,7 @@ private:
     bool m_selfWatch;
     bool m_reenter;
     bool m_setdata;
+    bool m_writing;
     int m_timeout;
     bool m_timebomb;
     bool m_restart;
@@ -312,14 +317,32 @@ public:
 class ExtModCommand : public MessageHandler
 {
 public:
-    ExtModCommand(const char *name)
-	: MessageHandler(name,100,__plugin.name())
+    ExtModCommand()
+	: MessageHandler("engine.command",100,__plugin.name())
 	{ }
     virtual bool received(Message &msg);
 private:
     bool complete(const String& partLine, const String& partWord, String& rval) const;
 };
 
+class ExtModStatus : public MessageHandler
+{
+public:
+    ExtModStatus()
+	: MessageHandler("engine.status",110,__plugin.name())
+	{ }
+    virtual bool received(Message &msg);
+};
+
+class ExtModHelp : public MessageHandler
+{
+public:
+    ExtModHelp()
+	: MessageHandler("engine.help",100,__plugin.name())
+	{ }
+    virtual bool received(Message& msg);
+};
+
 class ExtListener : public Thread
 {
 public:
@@ -741,7 +764,8 @@ ExtModReceiver::ExtModReceiver(const char* script, const char* args, File* ain,
     : Mutex(true,"ExtModReceiver"),
       m_role(RoleUnknown), m_dead(false), m_use(1), m_pid(-1),
       m_in(0), m_out(0), m_ain(ain), m_aout(aout),
-      m_chan(chan), m_watcher(0), m_selfWatch(false), m_reenter(false), m_setdata(true),
+      m_chan(chan), m_watcher(0),
+      m_selfWatch(false), m_reenter(false), m_setdata(true), m_writing(false),
       m_timeout(s_timeout), m_timebomb(s_timebomb), m_restart(false),
       m_script(script), m_args(args), m_trackName(s_trackName)
 {
@@ -758,7 +782,8 @@ ExtModReceiver::ExtModReceiver(const char* name, Stream* io, ExtModChan* chan, i
     : Mutex(true,"ExtModReceiver"),
       m_role(role), m_dead(false), m_use(1), m_pid(-1),
       m_in(io), m_out(io), m_ain(0), m_aout(0),
-      m_chan(chan), m_watcher(0), m_selfWatch(false), m_reenter(false), m_setdata(true),
+      m_chan(chan), m_watcher(0),
+      m_selfWatch(false), m_reenter(false), m_setdata(true), m_writing(false),
       m_timeout(s_timeout), m_timebomb(s_timebomb), m_restart(false),
       m_script(name), m_trackName(s_trackName)
 {
@@ -784,34 +809,28 @@ ExtModReceiver::~ExtModReceiver()
     if (m_pid > 1)
 	Debug(DebugWarn,"ExtModReceiver::~ExtModReceiver() pid=%d [%p]",m_pid,this);
     closeAudio();
+    Stream* tmp = m_in;
+    m_in = 0;
+    if (tmp == m_out)
+	m_out = 0;
+    delete tmp;
+    tmp = m_out;
+    m_out = 0;
+    delete tmp;
 }
 
 void ExtModReceiver::closeIn()
 {
-    if (!m_in)
-	return;
-    lock();
     Stream* tmp = m_in;
-    m_in = 0;
-    if (m_out == tmp)
-	m_out = 0;
-    unlock();
     if (tmp)
-	delete tmp;
+	tmp->terminate();
 }
 
 void ExtModReceiver::closeOut()
 {
-    if (!m_out)
-	return;
-    lock();
     Stream* tmp = m_out;
-    m_out = 0;
-    if (m_in == tmp)
-	m_in = 0;
-    unlock();
     if (tmp)
-	delete tmp;
+	tmp->terminate();
 }
 
 void ExtModReceiver::closeAudio()
@@ -938,7 +957,7 @@ bool ExtModReceiver::received(Message &msg, int id)
 	return false;
     lock();
     // check if we are no longer running
-    bool ok = (m_pid > 0) && m_in && m_out && !m_dead;
+    bool ok = (m_pid > 0) && !m_dead && m_in && m_in->valid() && m_out && m_out->valid();
     if (ok && !m_reenter) {
 	// check if the message was generated by ourselves - avoid reentrance
 	ExtMessage* m = YOBJECT(ExtMessage,&msg);
@@ -1041,7 +1060,7 @@ bool ExtModReceiver::create(const char *script, const char *args)
 	else
 	    ::close(STDERR_FILENO+2);
 	// Blindly close everything but stdin/out/err/audio
-	for (x=STDERR_FILENO+3;x<1024;x++) 
+	for (x=STDERR_FILENO+3;x<1024;x++)
 	    ::close(x);
 	// Execute script
 	if (debugAt(DebugInfo))
@@ -1169,49 +1188,66 @@ void ExtModReceiver::run()
 
 bool ExtModReceiver::outputLine(const char* line)
 {
-    lock();
-    DDebug("ExtModReceiver",DebugAll,"%soutputLine '%s'",
-	((m_out && !m_dead) ? "" : "failing "), line);
-    if (m_dead || !m_out) {
-	unlock();
-	return false;
-    }
-    // since m_out can be non-blocking (the socket) we have to loop
+    if (TelEngine::null(line))
+	return true;
     int len = ::strlen(line);
-    while (m_out && (len > 0) && !m_dead) {
+    if (m_dead || !m_out || !m_out->valid() || !use())
+	return false;
+    uint64_t tout = (m_timeout > 0) ? (Time::now() + 1000 * (uint64_t)m_timeout) : 0;
+    for (;;) {
+	Lock mylock(this);
+	if (m_dead || !m_out || !m_out->valid()) {
+	    unuse();
+	    return false;
+	}
+	if (!m_writing) {
+	    m_writing = true;
+	    break;
+	}
+	if (tout && tout < Time::now()) {
+	    Alarm("extmodule","performance",DebugWarn,"Timeout %d msec for %d characters [%p]",
+		m_timeout,len,this);
+	    unuse();
+	    return false;
+	}
+	mylock.drop();
+	Thread::idle();
+    }
+    bool ok = outputLineInternal(line,len);
+    m_writing = false;
+    unuse();
+    return ok;
+}
+
+bool ExtModReceiver::outputLineInternal(const char* line, int len)
+{
+    DDebug("ExtModReceiver",DebugAll,"outputLine len=%d '%s' [%p]",len,line,this);
+    // since m_out can be non-blocking (the socket) we have to loop
+    while (m_out && m_out->valid() && (len > 0) && !m_dead) {
 	int w = m_out->writeData(line,len);
 	if (w < 0) {
-	    if (m_dead || !m_out || !m_out->canRetry()) {
-		unlock();
+	    if (m_dead || !m_out || !m_out->canRetry())
 		return false;
-	    }
 	}
 	else {
 	    line += w;
 	    len -= w;
 	}
-	if (len > 0) {
-	    unlock();
+	if (len > 0)
 	    Thread::idle();
-	    lock();
-	}
     }
     char nl = '\n';
     for (;;) {
-	if (m_dead || !m_out) {
-	    unlock();
+	if (m_dead || !m_out)
 	    return false;
-	}
 	int w = m_out->writeData(&nl,1);
-	if ((w < 0) && m_out->canRetry())
+	if ((w < 0) && m_out && m_out->canRetry())
 	    w = 0;
-	unlock();
 	if (w > 0)
 	    return true;
 	if (w < 0)
 	    return false;
 	Thread::idle();
-	lock();
     }
 }
 
@@ -1224,7 +1260,8 @@ void ExtModReceiver::reportError(const char* line)
 void ExtModReceiver::returnMsg(const Message* msg, const char* id, bool accepted)
 {
     String ret(msg->encode(accepted,id));
-    outputLine(ret);
+    if (!outputLine(ret) && m_timebomb)
+	die();
 }
 
 bool ExtModReceiver::addWatched(const String& name)
@@ -1717,14 +1754,14 @@ bool ExtModCommand::complete(const String& partLine, const String& partWord, Str
 {
     if (partLine.null() && partWord.null())
 	return false;
-    if (partLine.null())
+    if (partLine.null() || partLine == YSTRING("status") || partLine == YSTRING("help"))
 	Module::itemComplete(rval,"external",partWord);
-    else if (partLine == "external") {
+    else if (partLine == YSTRING("external")) {
 	for (const char** list = s_cmds; *list; list++)
 	    Module::itemComplete(rval,*list,partWord);
 	return true;
     }
-    else if (partLine == "external restart" || partLine == "external stop") {
+    else if (partLine == YSTRING("external restart") || partLine == YSTRING("external stop")) {
 	ObjList mod;
 	s_mutex.lock();
 	ObjList *l = &s_modules;
@@ -1744,6 +1781,32 @@ bool ExtModCommand::complete(const String& partLine, const String& partWord, Str
 }
 
 
+bool ExtModStatus::received(Message& msg)
+{
+    const String& dest = msg[YSTRING("module")];
+    if (dest && (dest != YSTRING("external")))
+	return false;
+    s_mutex.lock();
+    msg.retValue() << "name=" << __plugin.name()
+	<< ",type=misc;scripts=" << s_modules.count()
+	<< ",chans=" << s_chans.count() << "\r\n";
+    s_mutex.unlock();
+    return !dest.null();
+}
+
+
+bool ExtModHelp::received(Message& msg)
+{
+    const String& line = msg[YSTRING("line")];
+    if (line && (line != YSTRING("external")))
+	return false;
+    msg.retValue() << "  " << s_helpExternalCmd << "\r\n";
+    if (line)
+	msg.retValue() << s_helpExternalInfo << "\r\n";
+    return !line.null();
+}
+
+
 ExtListener::ExtListener(const char* name)
     : Thread("ExtMod Listener"),
       m_name(name), m_role(ExtModReceiver::RoleUnknown)
@@ -1883,7 +1946,9 @@ void ExtModulePlugin::initialize()
     if (!m_handler) {
 	m_handler = new ExtModHandler("call.execute",s_cfg.getIntValue("general","priority",100));
 	Engine::install(m_handler);
-	Engine::install(new ExtModCommand("engine.command"));
+	Engine::install(new ExtModCommand);
+	Engine::install(new ExtModStatus);
+	Engine::install(new ExtModHelp);
 	NamedList *sect = 0;
 	int n = s_cfg.sections();
 	for (int i = 0; i < n; i++) {
