@@ -6,7 +6,7 @@
  * when their status changes
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2013 Null Team
+ * Copyright (C) 2004-2014 Null Team
  *
  * This software is distributed under multiple licenses;
  * see the COPYING file in the main directory for licensing
@@ -218,7 +218,7 @@ QueuedCall::QueuedCall(const String& queue, const String& chan, unsigned int sta
     int queuePrio, int callerPrio)
     : Mutex(true,"QueuedCall"),
     m_queue(queue), m_channelid(chan),
-    m_queuePrio(m_queuePrio), m_callerPrio(callerPrio),
+    m_queuePrio(queuePrio), m_callerPrio(callerPrio),
     m_startTime(start), m_caller(caller),
     m_called(called), m_callername(callername),
     m_destroyTime(0)
