@@ -5,7 +5,7 @@
  * Zaptel PRI/TDM/FXS/FXO cards signalling and data driver
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2013 Null Team
+ * Copyright (C) 2004-2014 Null Team
  *
  * This software is distributed under multiple licenses;
  * see the COPYING file in the main directory for licensing
@@ -102,7 +102,7 @@ class ZapModule;                         // The module
 
 #define DAHDI_EVENT_PULSEDIGIT 		ZT_EVENT_PULSEDIGIT
 #define DAHDI_EVENT_DTMFDOWN   		ZT_EVENT_DTMFDOWN
-#define DAHDI_EVENT_DTMFUP  		ZT_EVENT_DTMFUP 
+#define DAHDI_EVENT_DTMFUP  		ZT_EVENT_DTMFUP
 #define DAHDI_EVENT_PULSEDIGIT		ZT_EVENT_PULSEDIGIT
 
 // hook events
@@ -579,15 +579,13 @@ public:
 	const NamedList& params);
     virtual ~ZapCircuit()
 	{ cleanup(false); }
-    inline const ZapDevice device() const
-	{ return m_device; }
     virtual void destroyed()
 	{ cleanup(true); }
     // Change circuit status. Clear events on status change
     // New status is Connect: Open device. Create source/consumer. Start worker
     // Cleanup on disconnect
     virtual bool status(Status newStat, bool sync = false);
-    // Update data format for zaptel device and source/consumer 
+    // Update data format for zaptel device and source/consumer
     virtual bool updateFormat(const char* format, int direction);
     // Setup echo canceller or start echo canceller training
     virtual bool setParam(const String& param, const String& value);
@@ -983,7 +981,7 @@ static TokenDict s_formats[] = {
     {"alaw",    ZapDevice::Alaw},
     {0,0}
     };
-#ifdef HAVE_ZAP 
+#ifdef HAVE_ZAP
 const char* ZapDevice::s_zapCtlName = "/dev/zap/ctl";
 const char* ZapDevice::s_zapDevName = "/dev/zap/channel";
 #else
@@ -1404,7 +1402,7 @@ bool ZapDevice::checkAlarms()
 void ZapDevice::resetAlarms()
 {
     m_alarms = 0;
-    m_alarmsText = ""; 
+    m_alarmsText = "";
     Debug(m_owner,DebugInfo,"%sNo more alarms on channel %u [%p]",
 	m_name.safe(),m_channel,m_owner);
 }
@@ -1701,6 +1699,7 @@ SignallingComponent* ZapInterface::create(const String& type, NamedList& name)
     else
 	return 0;
 
+    TempObjectCounter cnt(plugin.objectsCounter());
     const String* module = name.getParam("module");
     if (module && *module != "zapcard")
 	return 0;
@@ -1794,6 +1793,7 @@ SignallingComponent* ZapInterface::create(const String& type, NamedList& name)
 bool ZapInterface::init(ZapDevice::Type type, unsigned int code, unsigned int channel,
 	const NamedList& config, const NamedList& defaults, const NamedList& params)
 {
+    TempObjectCounter cnt(plugin.objectsCounter());
     m_device.channel(channel,code);
     m_readOnly = getBoolValue("readonly",config,defaults,params);
     m_priority = Thread::priority(config.getValue("priority",defaults.getValue("priority")));
@@ -2007,6 +2007,7 @@ void ZapInterface::checkEvents()
 bool ZapSpan::init(ZapDevice::Type type, unsigned int offset,
 	const NamedList& config, const NamedList& defaults, const NamedList& params)
 {
+    TempObjectCounter cnt(plugin.objectsCounter());
     String voice = config.getValue("voicechans");
     unsigned int chans = 0;
     bool digital = true;
@@ -2215,6 +2216,7 @@ bool ZapCircuit::status(Status newStat, bool sync)
 	    code(),lookupStatus(newStat),this);
 	return false;
     }
+    TempObjectCounter cnt(plugin.objectsCounter());
     Status oldStat = SignallingCircuit::status();
     // Allow status change for the following values
     switch (newStat) {
@@ -2260,7 +2262,7 @@ bool ZapCircuit::status(Status newStat, bool sync)
     return false;
 }
 
-// Update data format for zaptel device and source/consumer 
+// Update data format for zaptel device and source/consumer
 bool ZapCircuit::updateFormat(const char* format, int direction)
 {
     s_sourceAccessMutex.lock();
@@ -2272,6 +2274,7 @@ bool ZapCircuit::updateFormat(const char* format, int direction)
     // Do nothing if format is the same
     if (src->getFormat() == format && m_consumer && m_consumer->getFormat() == format)
 	return false;
+    TempObjectCounter cnt(plugin.objectsCounter());
     // Check format
     // T1,E1: allow alaw or mulaw
     int f = lookup(format,s_formats,-2);
@@ -2306,6 +2309,7 @@ bool ZapCircuit::updateFormat(const char* format, int direction)
 // Setup echo canceller or start echo canceller training
 bool ZapCircuit::setParam(const String& param, const String& value)
 {
+    TempObjectCounter cnt(plugin.objectsCounter());
     if (param == "echotrain") {
 	int tmp = value.toInteger(-1);
 	if (tmp >= 0)
@@ -2329,7 +2333,7 @@ bool ZapCircuit::setParam(const String& param, const String& value)
 	bool ok = m_device.setEchoCancel(m_crtEchoCancel,m_echoTaps);
 	if (m_crtEchoCancel)
 	    m_crtEchoCancel = ok;
-	return ok; 
+	return ok;
     }
     if (param == "echotaps") {
 	int tmp = value.toInteger();
@@ -2358,6 +2362,7 @@ bool ZapCircuit::setParam(const String& param, const String& value)
 // Get circuit data
 bool ZapCircuit::getParam(const String& param, String& value) const
 {
+    TempObjectCounter cnt(plugin.objectsCounter());
     if (param == "buflen")
 	value = m_buflen;
     else if (param == "tonedetect")
@@ -2427,6 +2432,7 @@ bool ZapCircuit::sendEvent(SignallingCircuitEvent::Type type, NamedList* params)
     XDebug(group(),DebugAll,"ZapCircuit(%u). sendEvent(%u) [%p]",code(),type,this);
     if (!m_canSend)
 	return false;
+    TempObjectCounter cnt(plugin.objectsCounter());
 
     if (type == SignallingCircuitEvent::Dtmf) {
 	const char* tones = 0;
@@ -2647,6 +2653,7 @@ bool ZapAnalogCircuit::status(Status newStat, bool sync)
 	    code(),newStat,this);
 	return false;
     }
+    TempObjectCounter cnt(plugin.objectsCounter());
     // Allow status change for the following values
     switch (newStat) {
 	case Missing:
@@ -2727,6 +2734,7 @@ bool ZapAnalogCircuit::getParam(const String& param, String& value) const
 // Set line polarity
 bool ZapAnalogCircuit::setParam(const String& param, const String& value)
 {
+    TempObjectCounter cnt(plugin.objectsCounter());
     if (param == "polarity") {
 	if (!(m_device.valid() && value.isBoolean()))
 	    return false;
@@ -2745,6 +2753,7 @@ bool ZapAnalogCircuit::sendEvent(SignallingCircuitEvent::Type type, NamedList* p
     if (type == SignallingCircuitEvent::Dtmf)
 	return ZapCircuit::sendEvent(type,params);
 
+    TempObjectCounter cnt(plugin.objectsCounter());
     XDebug(group(),DebugAll,"ZapAnalogCircuit(%u). sendEvent(%u) [%p]",code(),type,this);
     switch (type) {
 	case SignallingCircuitEvent::OnHook:
