@@ -1,7 +1,21 @@
-/*
-    test.c
-    This file holds the entry point of the Telephony Engine
-*/
+/**
+ * test.cpp
+ * This file is part of the YATE Project http://YATE.null.ro
+ *
+ * Yet Another Telephony Engine - a fully featured software PBX and IVR
+ * Copyright (C) 2004-2014 Null Team
+ *
+ * This software is distributed under multiple licenses;
+ * see the COPYING file in the main directory for licensing
+ * information for this specific distribution.
+ *
+ * This use of this software may be subject to additional restrictions.
+ * See the LEGAL file in the main directory for details.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ */
 
 #include <yatengine.h>
 
@@ -41,20 +55,4 @@ void TestPlugin::initialize()
 
 INIT_PLUGIN(TestPlugin);
 
-/*
- * vim:ts=4:et:sw=4:ht=8
- *
- * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2013 Null Team
- *
- * This software is distributed under multiple licenses;
- * see the COPYING file in the main directory for licensing
- * information for this specific distribution.
- *
- * This use of this software may be subject to additional restrictions.
- * See the LEGAL file in the main directory for details.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- */
+/* vi: set ts=8 sw=4 sts=4 noet: */
