@@ -5,7 +5,7 @@
  * IAX channel
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2013 Null Team
+ * Copyright (C) 2004-2014 Null Team
  * Author: Marian Podgoreanu
  *
  * This software is distributed under multiple licenses;
@@ -434,6 +434,7 @@ public:
 protected:
     virtual bool commandComplete(Message& msg, const String& partLine, const String& partWord);
     virtual void msgStatus(Message& msg);
+    virtual void statusParams(String& str);
     void msgStatusAccounts(Message& msg);
     void msgStatusListeners(Message& msg);
     virtual void genUpdate(Message& msg);
@@ -1115,7 +1116,7 @@ void YIAXLineContainer::evTimer(Time& time)
 {
     lock();
     ListIterator iter(m_lines);
-    GenObject* gen = 0; 
+    GenObject* gen = 0;
     while (0 != (gen = iter.get())) {
 	RefPointer<YIAXLine> line = static_cast<YIAXLine*>(gen);
 	if (!line)
@@ -2198,6 +2199,18 @@ void YIAXDriver::msgStatus(Message& msg)
     Driver::msgStatus(msg);
 }
 
+void YIAXDriver::statusParams(String& str)
+{
+    Driver::statusParams(str);
+    m_enginesMutex.lock();
+    unsigned int cnt = 0;
+    ListIterator iter(m_engines);
+    while (YIAXEngine* eng = static_cast<YIAXEngine*>(iter.get()))
+	cnt += eng->transactionCount();
+    m_enginesMutex.unlock();
+    str.append("transactions=",",") << cnt;
+}
+
 void YIAXDriver::msgStatusAccounts(Message& msg)
 {
     msg.retValue().clear();
@@ -2220,7 +2233,7 @@ void YIAXDriver::msgStatusAccounts(Message& msg)
     }
     s_lines.unlock();
     msg.retValue() << "accounts=" << n;
-    msg.retValue().append(det,";"); 
+    msg.retValue().append(det,";");
     msg.retValue() << "\r\n";
 }
 
@@ -2257,7 +2270,7 @@ void YIAXDriver::msgStatusListeners(Message& msg)
     }
     m_enginesMutex.unlock();
     msg.retValue() << "listeners=" << n << ",default=" << def;
-    msg.retValue().append(det,";"); 
+    msg.retValue().append(det,";");
     msg.retValue() << "\r\n";
 }
 
@@ -2440,7 +2453,7 @@ unsigned long YIAXConsumer::Consume(const DataBlock& data, unsigned long tStamp,
 //
 // YIAXSource
 //
-YIAXSource::YIAXSource(YIAXConnection* conn, u_int32_t format, const char* formatText, int type) 
+YIAXSource::YIAXSource(YIAXConnection* conn, u_int32_t format, const char* formatText, int type)
     : DataSource(formatText),
     YIAXData(conn,format,type)
 {
@@ -2700,7 +2713,7 @@ void YIAXConnection::handleEvent(IAXEvent* event)
 	    startMedia(true,IAXFormat::Video);
 	    startMedia(false,IAXFormat::Video);
 	    Engine::enqueue(message("call.answered",false,true));
-	    break; 
+	    break;
 	case IAXEvent::Quelch:
 	    DDebug(this,DebugCall,"QUELCH [%p]",this);
 	    m_mutedOut = true;
@@ -2713,7 +2726,7 @@ void YIAXConnection::handleEvent(IAXEvent* event)
 	    DDebug(this,DebugInfo,"RINGING [%p]",this);
 	    startMedia(true);
 	    Engine::enqueue(message("call.ringing",false,true));
-	    break; 
+	    break;
 	case IAXEvent::Hangup:
 	case IAXEvent::Reject:
 	    {
